@@ -35,11 +35,8 @@ function UIBottomPanel:UIBottomPanel(ui)
   self.width = 640
   self.height = 48
   self:setDefaultPosition(0.5, -0.1)
-  self.panel_sprites = app.gfx:loadSpriteTable("Data", "Panel02V", true)
-  self.money_font = app.gfx:loadFont("QData", "Font05V")
-  self.date_font = app.gfx:loadFont("QData", "Font16V")
-  self.white_font = app.gfx:loadFont("QData", "Font01V", 0, -2)
-  self.pause_font = app.gfx:loadFont("QData", "Font124V")
+
+  self:_initFonts(app.gfx)
 
   -- State relating to fax notification messages
   self.show_animation = true
@@ -86,7 +83,7 @@ function UIBottomPanel:UIBottomPanel(ui)
   buttons[7] = panels[7]:makeToggleButton(1, 6, 35, 36, 28, self.dialogPolicy):setTooltip(_S.tooltip.toolbar.policy)
   for _, panel in ipairs(panels) do
     panel.visible = false
-    end
+  end
   self.additional_panels = panels
   self.additional_buttons = buttons
 
@@ -99,6 +96,16 @@ function UIBottomPanel:UIBottomPanel(ui)
   self:registerKeyHandlers()
 end
 
+function UIBottomPanel:_initFonts(gfx)
+  local date_label_color = { red = 175, green = 50, blue = 15 }
+  local pause_label_color = { red = 35, green = 138, blue = 173 }
+  self.panel_sprites = gfx:loadSpriteTable("Data", "Panel02V", true)
+  self.money_font = gfx:loadFontAndSpriteTable("QData", "Font05V")
+  self.date_font = gfx:loadFontAndSpriteTable("QData", "Font16V", nil, nil, 0, 0, date_label_color)
+  self.white_font = gfx:loadFontAndSpriteTable("QData", "Font01V", nil, nil, 0, -2)
+  self.pause_font = gfx:loadFontAndSpriteTable("QData", "Font124V", nil, nil, 0, 0, pause_label_color)
+end
+
 function UIBottomPanel:registerKeyHandlers()
   local ui = self.ui
   local buttons = self.additional_buttons
@@ -139,6 +146,7 @@ function UIBottomPanel:registerKeyHandlers()
   ui:addKeyHandler("ingame_openFirstMessage", self, self.openFirstMessage)    -- message
   ui:addKeyHandler("ingame_toggleInfo", self, self.toggleInformation)   -- information when you first build
   ui:addKeyHandler("ingame_jukebox", self, self.openJukebox)   -- jukebox
+  ui:addKeyHandler("ingame_panel_machineMenu", self, self.openMachineMenu)    -- machine menu
 end
 
 function UIBottomPanel:openJukebox()
@@ -147,6 +155,10 @@ function UIBottomPanel:openJukebox()
   end
 end
 
+function UIBottomPanel:openMachineMenu()
+  self.ui:addWindow(UIMachineMenu(self.ui))
+end
+
 function UIBottomPanel:openSave()
   self.ui:addWindow(UISaveGame(self.ui))
 end
@@ -177,7 +189,7 @@ function UIBottomPanel:draw(canvas, x, y)
   -- Draw balance with temporary offset in unicode languages
   x, y = x + self.x, y + self.y
   local offset_x, offset_y = 0, 0
-  if self.ui.app.gfx.language_font then
+  if self.ui.app.gfx:drawNumbersFromUnicode() then
     offset_x = 4
     offset_y = 2
   end
@@ -297,6 +309,7 @@ function UIBottomPanel:setDynamicInfo(info)
   end
 
   if not info then
+    -- dynamic info hide countdown value. 25 = ~1 second
     self.countdown = 25
   else
     self.countdown = nil
@@ -338,7 +351,7 @@ end
 -- appear on screen once there is space.
 --!param type (string) The type of message, can be: "emergency", "epidemy", "personality", "information", "disease", "report" or "strike"
 --!param message (table or number) If type == "strike", the amount of pay rise. Else a list of texts to display, including a "choices" table with choices. See below for structure.
---!param owner (humanoid or nil) Some messages are related to one staff or patient. Otherwise this is nil.
+--!param owner (object or nil) Some messages are related to one staff or patient or epidemic. Otherwise this is nil.
 --!param timeout (number or nil) If given, the message will expire after that many world ticks and be removed.
 --!param default_choice (number or nil) If given, the choice with this number will be executed on expiration of the message.
 --!param callback (function or nil) If given, it will be called when the message is closed.
@@ -882,7 +895,7 @@ function UIBottomPanel:afterLoad(old, new)
     end
   end
   if old < 58 then
-    self.pause_font = TheApp.gfx:loadFont("QData", "Font124V")
+    self.pause_font = TheApp.gfx:loadFontAndSpriteTable("QData", "Font124V")
   end
   if old < 62 then
     -- renamed additional_buttons to additional_panels
@@ -894,8 +907,11 @@ function UIBottomPanel:afterLoad(old, new)
     end
     self.bank_button = self.buttons[1]:makeToggle()
   end
+  if old < 215 then
+   self:_initFonts(self.ui.app.gfx)
+  end
   -- Hotfix to force re-calculation of the money font (see issue #1193)
-  self.money_font = self.ui.app.gfx:loadFont("QData", "Font05V")
+  self.money_font = self.ui.app.gfx:loadFontAndSpriteTable("QData", "Font05V")
 
   self:registerKeyHandlers()
 
