@@ -78,10 +78,10 @@ function UIEditRoom:UIEditRoom(ui, room_type)
     self.purchase_button:enable(true)
     self:checkEnableConfirm()
   end
-  self.blueprint_wall_anims = {
-  }
-  self.blueprint_window = {
-  }
+
+  self.blueprint_wall_anims = {}
+  self.blueprint_window = {}
+
   self.mouse_down_x = false
   self.mouse_down_y = false
   self.mouse_cell_x = 0
@@ -171,6 +171,7 @@ function UIEditRoom:cancel()
   if self.phase == "walls" then
     if self.paid then
       -- While the confirmation window is open, don't allow the player to click the confirm button.
+      local confirm_button_state = self.confirm_button.enabled
       self.confirm_button:enable(false)
       self.confirm_dialog_open = true
       -- Ask if the user really wish to sell this room
@@ -180,7 +181,7 @@ function UIEditRoom:cancel()
           self:abortRoom()
         end,
         --[[persistable:delete_room_confirm_dialog_cancel]]function()
-          self.confirm_button:enable(true)
+          self.confirm_button:enable(confirm_button_state)
           self.confirm_dialog_open = nil
         end
       ))
@@ -393,11 +394,25 @@ function UIEditRoom:finishRoom()
     flag = 1024
   end
   local function check_external_window(x, y, layer)
+    -- Test it is a wall, but not spr_num.
+    local function is_wall_and_not(wall_x, wall_y, spr_num)
+      return map:getCell(wall_x, wall_y, layer) ~= 0
+          and map:getCell(wall_x, wall_y, layer) ~= spr_num
+    end
+    -- Test that the wall is either spr_num1 or spr_num2.
+    local function is_one_of_wall(wall_x, wall_y, spr_num1, spr_num2)
+      return  map:getCell(wall_x, wall_y, layer) == spr_num1
+          or map:getCell(wall_x, wall_y, layer) == spr_num2
+    end
+
+
     -- If a wall is built which is normal to an external window, then said
-    -- window needs to be removed, otherwise it looks odd (see issue #59).
+    -- window needs to be removed, otherwise it looks odd.
     local block = map:getCell(x, y, layer)
     local dir = world:getWallDirFromBlockId(block)
     local tiles = self.ui.app.walls.external[world:getWallSetFromBlockId(block)]
+
+    -- Dispatch on the wall tiles at (x, y)
     if dir == "north_window_1" then
       if x ~= rect.x then
         map:setCell(x, y, layer, flag + tiles.north)
@@ -405,6 +420,7 @@ function UIEditRoom:finishRoom()
           map:setCell(x + 1, y, layer, flag + tiles.north)
         end
       end
+
     elseif dir == "north_window_2" then
       if x == rect.x then
         if map:getCell(x - 1, y, layer) ~= 0 then
@@ -412,6 +428,7 @@ function UIEditRoom:finishRoom()
         end
         map:setCell(x, y, layer, flag + tiles.north)
       end
+
     elseif dir == "west_window_1" then
       if y == rect.y then
         map:setCell(x, y, layer, flag + tiles.west)
@@ -419,6 +436,7 @@ function UIEditRoom:finishRoom()
           map:setCell(x, y - 1, layer, flag + tiles.west)
         end
       end
+
     elseif dir == "west_window_2" then
       if y ~= rect.y then
         if map:getCell(x, y + 1, layer) ~= 0 then
@@ -426,8 +444,58 @@ function UIEditRoom:finishRoom()
         end
         map:setCell(x, y, layer, flag + tiles.west)
       end
+
+    elseif dir == "north_window_long" then
+      if x == rect.x then
+        map:setCell(x, y, layer, flag + tiles.north_window_1)
+        -- Check if the window_long continues.
+        -- If yes, replace the window_long with window_n.
+        -- if no, replace it with the wall.
+        if is_wall_and_not(x - 1, y, tiles.north_window_2) then
+          if is_one_of_wall(x - 2, y,
+              tiles.north_window_long, tiles.north_window_1) then
+            map:setCell(x - 1, y, layer, flag + tiles.north_window_2)
+          else
+            map:setCell(x - 1, y, layer, flag + tiles.north)
+          end
+        end
+      else
+        map:setCell(x, y, layer, flag + tiles.north_window_2)
+        if is_wall_and_not(x + 1, y, tiles.north_window_1) then
+          if is_one_of_wall(x + 2, y,
+              tiles.north_window_long, tiles.north_window_2) then
+            map:setCell(x + 1, y, layer, flag + tiles.north_window_1)
+          else
+            map:setCell(x + 1, y, layer, flag + tiles.north)
+          end
+        end
+      end
+
+    elseif dir == "west_window_long" then
+      if y == rect.y then
+        map:setCell(x, y, layer, flag + tiles.west_window_2)
+        if is_wall_and_not(x, y - 1, tiles.west_window_1) then
+          if is_one_of_wall(x, y - 2,
+              tiles.west_window_long, tiles.west_window_2) then
+            map:setCell(x, y - 1, layer, flag + tiles.west_window_1)
+          else
+            map:setCell(x, y - 1, layer, flag + tiles.west)
+          end
+        end
+      else
+        map:setCell(x, y, layer, flag + tiles.west_window_1)
+        if is_wall_and_not(x, y + 1, tiles.west_window_2) then
+          if is_one_of_wall(x, y + 2,
+              tiles.west_window_long, tiles.west_window_1) then
+            map:setCell(x, y + 1, layer, flag + tiles.west_window_2)
+          else
+            map:setCell(x, y + 1, layer, flag + tiles.west)
+          end
+        end
+      end
     end
   end
+
   for x, obj in pairs(self.blueprint_wall_anims) do
     for y, anim in pairs(obj) do
       if x == rect.x and y == rect.y then
@@ -595,7 +663,7 @@ function UIEditRoom:returnToWallPhase(early)
 end
 
 -- Remove walls
-local function remove_wall_line(x, y, step_x, step_y, n_steps, layer, neigh_x, neigh_y, world)
+function UIEditRoom:_remove_wall_line(x, y, step_x, step_y, n_steps, layer, neigh_x, neigh_y, world)
   local map = world.map.th
   for _ = 1, n_steps do
     local existing = map:getCell(x, y, layer)
@@ -673,10 +741,10 @@ function UIEditRoom:removeRoom(save_objects, room, world)
     UIPlaceObjects.removeAllObjects(self, true)
   end
 
-  remove_wall_line(room.x, room.y, 0, 1, room.height, 3, -1,  0, world)
-  remove_wall_line(room.x, room.y, 1, 0, room.width , 2,  0, -1, world)
-  remove_wall_line(room.x + room.width, room.y , 0, 1, room.height, 3, 0, 0, world)
-  remove_wall_line(room.x, room.y + room.height, 1, 0, room.width , 2, 0, 0, world)
+  self:_remove_wall_line(room.x, room.y, 0, 1, room.height, 3, -1,  0, world)
+  self:_remove_wall_line(room.x, room.y, 1, 0, room.width , 2,  0, -1, world)
+  self:_remove_wall_line(room.x + room.width, room.y , 0, 1, room.height, 3, 0, 0, world)
+  self:_remove_wall_line(room.x, room.y + room.height, 1, 0, room.width , 2, 0, 0, world)
 
   -- Reset floor tiles and flags
   world.map.th:unmarkRoom(room.x, room.y, room.width, room.height)
@@ -714,7 +782,8 @@ function UIEditRoom:screenToWall(x, y)
   local rect = self.blueprint_rect
 
   if cellx == rect.x or cellx == rect.x - 1 or cellx == rect.x + rect.w or cellx == rect.x + rect.w - 1 or
-     celly == rect.y or celly == rect.y - 1 or celly == rect.y + rect.h or celly == rect.y + rect.h - 1 then
+     celly == rect.y or celly == rect.y - 1 or celly == rect.y + rect.h or
+     celly == rect.y + rect.h - 1 then -- luacheck: ignore 542
   else
     return
   end
@@ -1007,7 +1076,7 @@ function UIEditRoom:onMouseDown(button, x, y)
   if self.world.user_actions_allowed and not self.confirm_dialog_open then
     if button == "left" then
       if self.phase == "walls" then
-        if 0 <= x and x < self.width and 0 <= y and y < self.height then
+        if 0 <= x and x < self.width and 0 <= y and y < self.height then -- luacheck: ignore 542
         else
           local mouse_x, mouse_y = self.ui:ScreenToWorld(self.x + x, self.y + y)
           self.mouse_down_x = math.floor(mouse_x)
@@ -1015,9 +1084,7 @@ function UIEditRoom:onMouseDown(button, x, y)
           if self.move_rect then
             self.move_rect_x = self.mouse_down_x - self.blueprint_rect.x
             self.move_rect_y = self.mouse_down_y - self.blueprint_rect.y
-          elseif self.resize_rect then
-            -- nothing to do
-          else
+          elseif not self.resize_rect then
             self:setBlueprintRect(self.mouse_down_x, self.mouse_down_y, 1, 1)
           end
         end
@@ -1050,7 +1117,7 @@ function UIEditRoom:onMouseUp(button, x, y)
   return UIPlaceObjects.onMouseUp(self, button, x, y)
 end
 
-function UIEditRoom:onMouseMove(x, y, ...)
+function UIEditRoom:onMouseMove(x, y, dx, dy)
   if self.in_pickup_mode then
     self.ui:setCursor(self.ui.app.gfx:loadMainCursor("grab"))
   end
@@ -1555,6 +1622,15 @@ function UIEditRoom:placeObject()
 end
 
 function UIEditRoom:afterLoad(old, new)
+  if old < 172 then
+    -- reverts change in 171 where walls were moved into edit_room, its original
+    -- afterload was removed due to crashes
+    self.wall_types = nil
+    self.wall_id_by_block_id = nil
+    self.wall_set_by_block_id = nil
+    self.wall_dir_by_block_id = nil
+  end
+
   UIPlaceObjects.afterLoad(self, old, new)
   self:registerKeyHandlers()
 end
