@@ -51,18 +51,18 @@ function UIAnnualReport:UIAnnualReport(ui, world)
   self.rep_amount = 0
 
   if not pcall(function()
-    local palette   = gfx:loadPalette("QData", "Award02V.pal")
-    palette:setEntry(255, 0xFF, 0x00, 0xFF) -- Make index 255 transparent
+    local palette = gfx:loadPalette("QData", "Award02V.pal", true)
 
     -- Right now the statistics are first
     --self.background = gfx:loadRaw("Fame01V", 640, 480)
     self.award_background = gfx:loadRaw("Award01V", 640, 480)
-    self.stat_background = gfx:loadRaw("Award02V", 640, 480)
+    self.stat_background = gfx:loadRaw("Award02V", 640, 480, "QData", "QData", "Award02V.pal", true)
     self.background = self.stat_background
 
-    self.stat_font = gfx:loadFont("QData", "Font45V", false, palette)
-    self.write_font = gfx:loadFont("QData", "Font47V", false, palette)
-    self.stone_font = gfx:loadFont("QData", "Font46V", false, palette)
+    local stone_text_color = { red = 80, green = 60, blue = 35 }
+    self.stat_font = gfx:loadFontAndSpriteTable("QData", "Font45V", false, palette)
+    self.write_font = gfx:loadFontAndSpriteTable("QData", "Font47V", false, palette)
+    self.stone_font = gfx:loadFontAndSpriteTable("QData", "Font46V", false, palette, 0, 0, stone_text_color)
 
     self.panel_sprites = gfx:loadSpriteTable("QData", "Award03V", true, palette)
   end) then
@@ -175,6 +175,8 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
   local level_config = world.map.level_config
   local prices = level_config and level_config.awards_trophies or nil
 
+  local anybody_cured = hosp.num_cured_ty > 1
+
   -- Check CuresAward so that we know the new config settings are available
   if prices and prices.TrophyMayorBonus then
     self.won_amount = 0
@@ -190,29 +192,34 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
 
     -- Coke sales
     if hosp.sodas_sold > prices.CansofCoke then
+      -- Bonus
       self:addTrophy(_S.trophy_room.sold_drinks.trophies[math.random(1, 3)], "money", prices.CansofCokeBonus)
       self.won_amount = self.won_amount + prices.CansofCokeBonus
     end
     -- Impressive VIP visits
-    if  hosp.num_vips_ty > 0 and hosp.pleased_vips_ty == hosp.num_vips_ty then
-      -- added some here so you get odd amounts as in TH!
+    if anybody_cured and hosp.num_vips_ty > 0 and hosp.pleased_vips_ty == hosp.num_vips_ty then
+      -- Bonus
       local win_value = (prices.TrophyMayorBonus * hosp.pleased_vips_ty) + math.random(1, 5)
       self:addTrophy(_S.trophy_room.happy_vips.trophies[math.random(1, 3)], "reputation", win_value)
       self.rep_amount = self.rep_amount + win_value
     end
     -- Impressive Reputation in the year (above a threshold throughout the year)
-    if hosp.has_impressive_reputation then
+    if anybody_cured and hosp.has_impressive_reputation then
+      -- Bonus
       self:addTrophy(_S.trophy_room.consistant_rep.trophies[math.random(1, 2)], "money", prices.TrophyReputationBonus)
       self.won_amount = self.won_amount + prices.TrophyReputationBonus
     end
     -- Everyone treated successfully, no deaths, or around a 100% Cure rate in the year
-    if hosp.num_cured_ty > 1 and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+    if anybody_cured and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+      -- Bonus
       self:addTrophy(_S.trophy_room.all_cured.trophies[math.random(1, 2)], "money", prices.TrophyAllCuredBonus)
       self.won_amount = self.won_amount + prices.TrophyAllCuredBonus
-    elseif hosp.num_deaths_this_year == 0 then
+    elseif anybody_cured and hosp.num_deaths_this_year == 0 then
+      -- Bonus
       self:addTrophy(_S.trophy_room.no_deaths.trophies[math.random(1, 3)], "money", prices.TrophyDeathBonus)
       self.won_amount = self.won_amount + prices.TrophyDeathBonus
-    elseif hosp.num_cured_ty > (hosp.not_cured_ty * 0.9)  then
+    elseif anybody_cured and hosp.num_cured_ty > (hosp.not_cured_ty * 0.9) then
+      -- Bonus
       self:addTrophy(_S.trophy_room.many_cured.trophies[math.random(1, 3)], "money", prices.TrophyCuresBonus)
       self.won_amount = self.won_amount + prices.TrophyCuresBonus
     end
@@ -222,22 +229,24 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
 
 
     -- Reputation
-    if hosp.reputation > prices.ReputationAward then
+    if anybody_cured and hosp.reputation > prices.ReputationAward then
+      -- Bonus
       self:addAward(_S.trophy_room.high_rep.awards[math.random(1, 2)], "money", prices.AwardReputationBonus)
       self.award_won_amount = self.award_won_amount + prices.AwardReputationBonus
     elseif hosp.reputation < prices.ReputationPoor then
+      -- Penalty
       self:addAward(_S.trophy_room.high_rep.penalty[math.random(1, 2)], "money", prices.AwardReputationPenalty)
       self.award_won_amount = self.award_won_amount + prices.AwardReputationPenalty
     end
 
     -- Hospital Value
-    if hosp.value > prices.HospValueAward then
-      -- added some here so you get odd amounts as in TH!
+    if anybody_cured and hosp.value > prices.HospValueAward then
+      -- Bonus
       local win_value = prices.HospValueBonus * math.random(1, 15)
       self:addAward(_S.trophy_room.hosp_value.awards[1], "reputation", win_value)
       self.rep_amount = self.rep_amount + win_value
     elseif hosp.value < prices.HospValuePoor then
-      -- added some here so you get odd amounts as in TH!
+      -- Penalty
       local lose_value = prices.HospValuePenalty * math.random(1, 15)
       self:addAward(_S.trophy_room.hosp_value.penalty[1], "reputation", lose_value)
       self.rep_amount = self.rep_amount + lose_value
@@ -246,22 +255,27 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
     -- Should these next few be linked so that you can only get one or should you get more than one if you met the targets?
 
     -- Cures
-    if hosp.num_cured_ty > prices.CuresAward then
+    if anybody_cured and hosp.num_cured_ty > prices.CuresAward then
+      -- Bonus
       self:addAward(_S.trophy_room.many_cured.awards[math.random(1, 2)], "money", prices.CuresBonus)
       self.award_won_amount = self.award_won_amount + prices.CuresBonus
     elseif hosp.num_cured_ty < prices.CuresPoor then
+      -- Penalty
       self:addAward(_S.trophy_room.many_cured.penalty[math.random(1, 2)], "money", prices.CuresPenalty)
       self.award_won_amount = self.award_won_amount + prices.CuresPenalty
     end
 
     -- Deaths
-    if hosp.num_cured_ty > 1 and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+    if anybody_cured and hosp.num_deaths_this_year == 0 and hosp.not_cured_ty == 0 then
+      -- Bonus
       self:addAward(_S.trophy_room.no_deaths.awards[1], "money", prices.AllCuresBonus)
       self.award_won_amount = self.award_won_amount + prices.AllCuresBonus
-    elseif hosp.num_deaths_this_year < prices.DeathsAward then
+    elseif anybody_cured and hosp.num_deaths_this_year < prices.DeathsAward then
+      -- Bonus
       self:addAward(_S.trophy_room.no_deaths.awards[math.random(1, 2)], "money", prices.DeathsBonus)
       self.award_won_amount = self.award_won_amount + prices.DeathsBonus
     elseif hosp.num_deaths_this_year > prices.DeathsPoor then
+      -- Penalty
       self:addAward(_S.trophy_room.no_deaths.penalty[math.random(1, 2)], "money", prices.DeathsPenalty)
       self.award_won_amount = self.award_won_amount + prices.DeathsPenalty
     end
@@ -272,10 +286,12 @@ function UIAnnualReport:checkTrophiesAndAwards(world)
     if hosp.num_deaths_this_year > 0 then
       cure_ratio = hosp.num_cured_ty / hosp.num_deaths_this_year
     end
-    if cure_ratio > prices.CuresVDeathsAward then
+    if anybody_cured and cure_ratio > prices.CuresVDeathsAward then
+      -- Bonus
       self:addAward(_S.trophy_room.curesvdeaths.awards[1], "money", prices.CuresVDeathsBonus)
       self.award_won_amount = self.award_won_amount + prices.CuresVDeathsBonus
     elseif cure_ratio <= prices.CuresVDeathsPoor then
+      -- Penalty
       self:addAward(_S.trophy_room.curesvdeaths.penalty[1], "money", prices.CuresVDeathsPenalty)
       self.award_won_amount = self.award_won_amount + prices.CuresVDeathsPenalty
     end
@@ -453,7 +469,6 @@ end
 function UIAnnualReport:close()
   if TheApp.world:getLocalPlayerHospital().game_won then
     TheApp.video:setBlueFilterActive(false)
-    TheApp.world.ui.bottom_panel:openLastMessage()
   end
   self:updateAwards()
   Window.close(self)
@@ -502,7 +517,12 @@ function UIAnnualReport:draw(canvas, x, y)
   local font = self.stat_font
   local world = self.ui.app.world
 
-  if self.state == 1 then -- Fame screen
+  if self.state == 1 then -- Fame/Shame screen (High Scores)
+    -- TODO: This screen should be displayed at the start of the annual report, but currently
+    -- it is not shown, likely as the code for this being unfinished. When implemented
+    -- we only show this screen at the very start, and once they go to the next screen
+    -- it no longer becomes accessible.
+
     -- Title and column names
     font:draw(canvas, _S.high_score.best_scores, x + 220, y + 104, 200, 0)
     font:draw(canvas, _S.high_score.pos, x + 218, y + 132)
@@ -514,7 +534,7 @@ function UIAnnualReport:draw(canvas, x, y)
     local dy = 0
     --for i = 1, 10 do
       font:draw(canvas, i .. ".", x + 220, y + 160 + dy)
-      font:draw(canvas, world.hospitals[1].name:upper(), x + 260, y + 160 + dy)
+      font:draw(canvas, world:getLocalPlayerHospital().name:upper(), x + 260, y + 160 + dy)
       font:draw(canvas, "NA", x + 360, y + 160 + dy)
       -- dy = dy + 25
     --end
@@ -632,3 +652,20 @@ function UIAnnualReport:drawStatisticsScreen(canvas, x, y)
         y + row_y + row_no_y * 2 + row_dy * (index_v2 - 1), 70, 0, "right")
   end
 end
+
+function UIAnnualReport:afterLoad(old, new)
+  if old < 179 then
+    local gfx = TheApp.gfx
+
+    local palette = gfx:loadPalette("QData", "Award02V.pal", true)
+    self.award_background = gfx:loadRaw("Award01V", 640, 480)
+    self.stat_background = gfx:loadRaw("Award02V", 640, 480, "QData", "QData", "Award02V.pal", true)
+    self.background = self.stat_background
+    self.stat_font = gfx:loadFontAndSpriteTable("QData", "Font45V", false, palette)
+    self.write_font = gfx:loadFontAndSpriteTable("QData", "Font47V", false, palette)
+    self.stone_font = gfx:loadFontAndSpriteTable("QData", "Font46V", false, palette)
+    self.panel_sprites = gfx:loadSpriteTable("QData", "Award03V", true, palette)
+  end
+
+  UIFullscreen.afterLoad(self, old, new)
+end
