@@ -33,7 +33,7 @@ function UIStaffManagement:UIStaffManagement(ui)
     self.background = gfx:loadRaw("Staff01V", 640, 480, "QData", "QData", "Staff01V.pal", true)
     local palette = gfx:loadPalette("QData", "Staff01V.pal", true)
     self.panel_sprites = gfx:loadSpriteTable("QData", "Staff02V", true, palette)
-    self.title_font = gfx:loadFont("QData", "Font01V", false, palette)
+    self.title_font = gfx:loadFontAndSpriteTable("QData", "Font01V", false, palette)
     self.face_parts = ui.app.gfx:loadRaw("Face01V", 65, 1350, nil, "Data", "MPalette.dat")
   end) then
     ui:addWindow(UIInformation(ui, {_S.errors.dialog_missing_graphics}))
@@ -419,22 +419,34 @@ end
 
 function UIStaffManagement:onMouseDown(code, x, y)
   if code == "left" then
-    if x > 50 and x < 490 then
-      if y > 82 and y < 351 then
-        if #self.staff_members[self.category] - (self.page - 1)*10 > math_floor((y - 81)/27) then
-          self.selected_staff = math_floor((y - 81)/27) + 1 + (self.page - 1)*10
-        end
+    local inside_staff_list_area = (x > 50 and x < 624) and (y > 82 and y < 351)
+    if inside_staff_list_area then
+      -- Hit staff row
+      if #self.staff_members[self.category] - (self.page - 1)*10 > math_floor((y - 81)/27) then
+        self.selected_staff = math_floor((y - 81)/27) + 1 + (self.page - 1)*10
       end
-    elseif x > 497 and x < 580 and y > 373 and y < 455 and self.selected_staff then
-      -- Hit in the view of the staff
+    else
+      local inside_view_of_the_staff_area = (x > 497 and x < 580) and (y > 373 and y < 455)
+      if inside_view_of_the_staff_area and self.selected_staff then
+        return false -- on false window dragging won't work
+      end
+    end
+  end
+  return UIFullscreen.onMouseDown(self, code, x, y)
+end
+
+function UIStaffManagement:onMouseUp(code, x, y)
+  if code == "left" then
+    local inside_view_of_the_staff_area = (x > 497 and x < 580) and (y > 373 and y < 455)
+    if inside_view_of_the_staff_area and self.selected_staff then
+      -- Hit in the view of the staff.
       local ui = self.ui
       ui:scrollMapTo(self:getStaffPosition())
       ui:addWindow(UIStaff(ui, self.staff_members[self.category][self.selected_staff]))
       self:close()
-      return false
     end
   end
-  return UIFullscreen.onMouseDown(self, code, x, y)
+  return UIFullscreen.onMouseUp(self, code, x, y)
 end
 
 function UIStaffManagement:onMouseWheel(x, y)
@@ -457,7 +469,7 @@ end
 function UIFullscreen:getStaffPosition(dx, dy)
   local staff = self.staff_members[self.category][self.selected_staff]
   local x, y = self.ui.app.map:WorldToScreen(staff.tile_x, staff.tile_y)
-  local px, py = staff.th:getMarker()
+  local px, py = staff.th:getSecondaryMarker()
   return x + px - (dx or 0), y + py - (dy or 0)
 end
 
@@ -549,7 +561,6 @@ function UIStaffManagement:increaseSalary()
   if self.selected_staff then
     local staff = self.staff_members[self.category][self.selected_staff]
     if staff:increaseWage(math_floor(staff.profile.wage * 0.1)) then
-      self.ui:playSound("bonusal2.wav")
       return
     end
   end
@@ -591,7 +602,7 @@ function UIStaffManagement:afterLoad(old, new)
     self.background = gfx:loadRaw("Staff01V", 640, 480, "QData", "QData", "Staff01V.pal", true)
     local palette = gfx:loadPalette("QData", "Staff01V.pal", true)
     self.panel_sprites = gfx:loadSpriteTable("QData", "Staff02V", true, palette)
-    self.title_font = gfx:loadFont("QData", "Font01V", false, palette)
+    self.title_font = gfx:loadFontAndSpriteTable("QData", "Font01V", false, palette)
   end
 
   UIFullscreen.afterLoad(self, old, new)
