@@ -30,6 +30,12 @@ local col_bg = {
   blue = 198,
 }
 
+local label_ttf_col = { -- off white
+  red = 250,
+  green = 249,
+  blue = 245,
+}
+
 local menu_item_height = 40
 
 function UIMainMenu:UIMainMenu(ui)
@@ -62,7 +68,8 @@ function UIMainMenu:UIMainMenu(ui)
   -- Work out the menu's height, giving extra space for the version information
   local line_height = 15
   local top_padding = 20
-  local num_lines = computeSize({TheApp:isUpdateCheckDisabledByConfig(), TheApp.config.debug, TheApp:getVersion()}) -- see UIMainMenu:draw for how these are used
+  local release_string = TheApp:getReleaseString()
+  local num_lines = computeSize({TheApp:isUpdateCheckDisabledByConfig(), TheApp.config.debug, release_string}) -- see UIMainMenu:draw for how these are used
   local bottom_text_height = (line_height * num_lines)
   self.height = top_padding + ((menu_item_height + 10) * #menu_items) + bottom_text_height
 
@@ -74,8 +81,8 @@ function UIMainMenu:UIMainMenu(ui)
   self:setDefaultPosition(0.5, 0.25)
 
   -- The main menu also shows the version number of the player's copy of the game.
-  self.label_font = TheApp.gfx:loadFont("QData", "Font01V")
-  self.version_number = TheApp:getVersion()
+  self.label_font = TheApp.gfx:loadFontAndSpriteTable("QData", "Font01V", nil, nil, 0, 0, label_ttf_col)
+  self.release_string = release_string
 
   -- individual buttons
   self.default_button_sound = "selectx.wav"
@@ -120,7 +127,10 @@ function UIMainMenu:draw(canvas, x, y)
     self.label_font:draw(canvas, _S.main_menu.savegame_version .. TheApp.savegame_version, x + 5, ly, 190, 0, "right")
     ly = ly - 15
   end
-  self.label_font:draw(canvas, _S.main_menu.version .. self.version_number, x + 5, ly, 190, 0, "right")
+  self.label_font:draw(canvas, _S.main_menu.version .. self.release_string, x + 5, ly, 190, 0, "right")
+end
+
+function UIMainMenu:onTick()
   TheApp:idle()
 end
 
