@@ -94,11 +94,8 @@ function UIWatch:onCountdownEnd()
     self.ui.hospital:resolveEmergency()
   elseif self.count_type == "epidemic" then
     local epidemic = self.hospital.epidemic
-    if epidemic and not epidemic.inspector then
-      epidemic:spawnInspector()
-      if epidemic.vaccination_mode_active then
-        epidemic:toggleVaccinationMode()
-      end
+    if epidemic then
+      epidemic:coverUpTimeIsUp()
     end
   elseif self.count_type == "initial_opening" then
     self.hospital:open()
