@@ -39,7 +39,7 @@ local check_watch_animations = permanent"humanoid_check_watch_animations"({})
 
 local mood_icons = permanent"humanoid_mood_icons"({})
 
-local function anims(name, walkN, walkE, idleN, idleE, doorL, doorE, knockN, knockE, swingL, swingE)
+local function walk_anims(name, walkN, walkE, idleN, idleE, doorL, doorE, knockN, knockE, swingL, swingE)
   walk_animations[name] = {
     walk_east = walkE,
     walk_north = walkN,
@@ -104,41 +104,91 @@ end
 local function pee_anim(name, peeAnim)
   pee_animations[name] = peeAnim
 end
-local function moods(name, iconNo, prio, alwaysOn)
-  mood_icons[name] = {icon = iconNo, priority = prio, on_hover = alwaysOn}
+local function moods(name, iconNo, prio, onHover)
+  mood_icons[name] = {icon = iconNo, priority = prio, on_hover = onHover}
 end
 
---   | Walk animations           |
---   | Name                      |WalkN|WalkE|IdleN|IdleE|DoorL|DoorE|KnockN|KnockE|SwingL|SwingE| Notes
------+---------------------------+-----+-----+-----+-----+-----+-----+------+------+-------+---------+
-anims("Standard Male Patient",       16,   18,   24,   26,  182,  184,   286,   288,  2040,  2042) -- 0-16, ABC
-anims("Gowned Male Patient",        406,  408,  414,  416)                           -- 0-10
-anims("Stripped Male Patient",      818,  820,  826,  828)                           -- 0-16
-anims("Stripped Male Patient 2",      818,  820,  826,  828)                           -- 0-16
-anims("Stripped Male Patient 3",      818,  820,  826,  828)
-anims("Alternate Male Patient",    2704, 2706, 2712, 2714, 2748, 2750,  2764,  2766) -- 0-10, ABC
-anims("Slack Male Patient",        1484, 1486, 1492, 1494, 1524, 1526,  2764,  1494) -- 0-14, ABC
-anims("Slack Female Patient",         0,    2,    8,   10,  258,  260,   294,   296,  2864,  2866) -- 0-16, ABC
-anims("Transparent Male Patient",  1064, 1066, 1072, 1074, 1104, 1106,  1120,  1074) -- 0-16, ABC
-anims("Standard Female Patient",      0,    2,    8,   10,  258,  260,   294,   296,  2864,  2866) -- 0-16, ABC
-anims("Gowned Female Patient",     2876, 2878, 2884, 2886)                           -- 0-8
-anims("Stripped Female Patient",    834,  836,  842,  844)                           -- 0-16
-anims("Stripped Female Patient 2",    834,  836,  842,  844)                           -- 0-16
-anims("Stripped Female Patient 3",    834,  836,  842,  844)
-anims("Transparent Female Patient",3012, 3014, 3020, 3022, 3052, 3054,  3068,  3070) -- 0-8, ABC
-anims("Chewbacca Patient",          858,  860,  866,  868, 3526, 3528,  4150,  4152)
-anims("Elvis Patient",              978,  980,  986,  988, 3634, 3636,  4868,  4870)
-anims("Invisible Patient",         1642, 1644, 1840, 1842, 1796, 1798,  4192,  4194)
-anims("Alien Male Patient",        3598, 3600, 3606, 3608,  182,  184,   286,   288, 3626,  3628) -- remember, no "normal"-doors animation
-anims("Alien Female Patient",      3598, 3600, 3606, 3608,  258,  260,   294,   296, 3626,  3628) -- identical to male; however death animations differ
-anims("Doctor",                      32,   34,   40,   42,  670,  672,   nil,   nil, 4750,  4752)
-anims("Surgeon",                   2288, 2290, 2296, 2298)
-anims("Nurse",                     1206, 1208, 1650, 1652, 3264, 3266,   nil,   nil, 3272,  3274)
-anims("Handyman",                  1858, 1860, 1866, 1868, 3286, 3288,   nil,   nil, 3518,  3520)
-anims("Receptionist",              3668, 3670, 3676, 3678) -- Could do with door animations
-anims("VIP",                        266,  268,  274,  276)
-anims("Inspector",                  266,  268,  274,  276)
-anims("Grim Reaper",                994,  996, 1002, 1004)
+--! Filter animations for patients and set the give marker positions.
+local function assignPatientMarkers(anims, name, ...)
+  local anim_mgr = TheApp.animation_manager
+
+  for hum_type, anim in pairs(anims) do
+    if string.find(hum_type, "Patient") then
+      if name then anim = anim[name] end
+      anim_mgr:setPatientMarker(anim, ...)
+    end
+  end
+end
+
+--! Filter animations for staff and set the give marker positions.
+local function assignStaffMarkers(anims, name, ...)
+  local anim_mgr = TheApp.animation_manager
+
+  for hum_type, anim in pairs(anims) do
+    if not string.find(hum_type, "Patient") then
+      if name then anim = anim[name] end
+      anim_mgr:setStaffMarker(anim, ...)
+    end
+  end
+end
+
+
+--   | Walk animations                 |
+--   | Name                            |WalkN|WalkE|IdleN|IdleE|DoorL|DoorE|KnockN|KnockE|SwingL|SwingE| Notes
+-----+---------------------------------------+-----+-----+-----+-----+-----+-----+------+------+-------+---------+
+walk_anims("Standard Male Patient",       16,   18,   24,   26,  182,  184,   286,   288,  2040,  2042) -- 0-16, ABC
+walk_anims("Gowned Male Patient",        406,  408,  414,  416)                           -- 0-10
+walk_anims("Stripped Male Patient",      818,  820,  826,  828)                           -- 0-16
+walk_anims("Stripped Male Patient 2",    818,  820,  826,  828)                           -- 0-16
+walk_anims("Stripped Male Patient 3",    818,  820,  826,  828)
+walk_anims("Alternate Male Patient",    2704, 2706, 2712, 2714, 2748, 2750,  2764,  2766) -- 0-10, ABC
+walk_anims("Slack Male Patient",        1484, 1486, 1492, 1494, 1524, 1526,  2764,  1494) -- 0-14, ABC
+walk_anims("Slack Female Patient",         0,    2,    8,   10,  258,  260,   294,   296,  2864,  2866) -- 0-16, ABC
+walk_anims("Transparent Male Patient",  1064, 1066, 1072, 1074, 1104, 1106,  1120,  1074) -- 0-16, ABC
+walk_anims("Standard Female Patient",      0,    2,    8,   10,  258,  260,   294,   296,  2864,  2866) -- 0-16, ABC
+walk_anims("Gowned Female Patient",     2876, 2878, 2884, 2886)                           -- 0-8
+walk_anims("Stripped Female Patient",    834,  836,  842,  844)                           -- 0-16
+walk_anims("Stripped Female Patient 2",  834,  836,  842,  844)                           -- 0-16
+walk_anims("Stripped Female Patient 3",  834,  836,  842,  844)
+walk_anims("Transparent Female Patient",3012, 3014, 3020, 3022, 3052, 3054,  3068,  3070) -- 0-8, ABC
+walk_anims("Chewbacca Patient",          858,  860,  866,  868, 3526, 3528,  4150,  4152)
+walk_anims("Elvis Patient",              978,  980,  986,  988, 3634, 3636,  4868,  4870)
+walk_anims("Invisible Patient",         1642, 1644, 1840, 1842, 1796, 1798,  4192,  4194)
+walk_anims("Alien Male Patient",        3598, 3600, 3606, 3608,  182,  184,   286,   288, 3626,  3628) -- remember, no "normal"-doors animation
+walk_anims("Alien Female Patient",      3598, 3600, 3606, 3608,  258,  260,   294,   296, 3626,  3628) -- identical to male; however death animations differ
+walk_anims("Doctor",                      32,   34,   40,   42,  670,  672,   nil,   nil, 4750,  4752)
+walk_anims("Surgeon",                   2288, 2290, 2296, 2298)
+walk_anims("Nurse",                     1206, 1208, 1650, 1652, 3264, 3266,   nil,   nil, 3272,  3274)
+walk_anims("Handyman",                  1858, 1860, 1866, 1868, 3286, 3288,   nil,   nil, 3518,  3520)
+walk_anims("Receptionist",              3668, 3670, 3676, 3678) -- Could do with door animations
+walk_anims("VIP",                        266,  268,  274,  276)
+walk_anims("Inspector",                  266,  268,  274,  276)
+walk_anims("Grim Reaper",                994,  996, 1002, 1004)
+
+
+local kfp1, kfp2, kfp3, kfp4, kfp5, kfp6, kfp7, kfp8
+kfp1, kfp2, kfp3, kfp4 = {-19, -15, "px"}, {-26, -17, "px"}, {-26, -15, "px"}, {-22, -13, "px"}
+kfp5, kfp6, kfp7, kfp8 = {-19, -9, "px"}, {-13, -5, "px"}, {-8, -4, "px"}, {-5, 0, "px"}
+local doorE_markers = { -- Anim 3288 in particular
+  kfp1, kfp2, kfp2, kfp2, kfp3, kfp4, kfp5, kfp6, kfp7, kfp8,
+}
+
+local kfp9, kfp10
+kfp1, kfp2, kfp3, kfp4, kfp5 = {-7, 0, "px"}, {-14, 6, "px"}, {-12, 5, "px"}, {-11, 5, "px"}, {-7, 3, "px"}
+kfp6, kfp7, kfp8, kfp9, kfp10 = {-4, 0, "px"}, {2, -1, "px"}, {10, -3, "px"}, {14, -7, "px"}, {20, -11, "px"}
+local doorL_markers = { -- Anim 3286 in particular.
+  kfp1, kfp2, kfp2, kfp3, kfp4, kfp5, kfp6, kfp7, kfp7, kfp8, kfp9, kfp10,
+}
+
+assignPatientMarkers(door_animations, "entering", doorE_markers)
+assignPatientMarkers(door_animations, "leaving", doorL_markers)
+assignPatientMarkers(door_animations, "leaving_swing", {3, 4, "px"}, {32, -14, "px"})
+assignPatientMarkers(door_animations, "entering_swing", {-32, -10, "px"}, {0, -2, "px"})
+
+assignStaffMarkers(door_animations, "entering", doorE_markers)
+assignStaffMarkers(door_animations, "leaving", doorL_markers)
+assignStaffMarkers(door_animations, "leaving_swing", {3, 4, "px"}, {32, -14, "px"})
+assignStaffMarkers(door_animations, "entering_swing", {-32, -10, "px"}, {0, -2, "px"})
 
 --  | Die Animations                 |
 --  | Name                           |FallE|RiseE|RiseE Hell|WingsE|HandsE|FlyE|ExtraE| Notes 2248
@@ -157,10 +207,11 @@ die_anims("Invisible Patient",         4200, 2434,       384, 2438,  2446, 2450)
 die_anims("Alien Male Patient",        4882, 2434,       384, 2438,  2446, 2450)
 die_anims("Alien Female Patient",      4886, 3208,       580, 3212,  3216, 3220)
 
--- The next fours sets belong together, but are done like this so we can use them on there own
--- I also had difficulty in keeping them together, as the patient needs to on the floor
+-- The next fours sets belong together, but are done like this so we can use them on their own
+-- I also had difficulty in keeping them together, as the patient needs to be on the floor
 -- for the duration of the earth quake before getting back up
 -- Shaking of fist could perhaps be used when waiting too long
+
 --  | Falling Animations                   |
 --  | Name                                 |Anim| Notes
 ----+--------------------------------+-----+-----+-----+-----+------+------+
@@ -170,8 +221,8 @@ falling_anim("Standard Female Patient",   3116)
 --  | On_ground Animations                   |
 --  | Name                                 |Anim| Notes
 ----+--------------------------------+-----+-----+-----+-----+------+------+
-on_ground_anim("Standard Male Patient",     1258)
-on_ground_anim("Standard Female Patient",   1764)
+on_ground_anim("Standard Male Patient",   1258)
+on_ground_anim("Standard Female Patient", 1764)
 
 --  | Get_up Animations                   |
 --  | Name                                 |Anim| Notes
@@ -182,8 +233,9 @@ get_up_anim("Standard Female Patient",   580)
 --  | Shake_fist Animations                   |
 --  | Name                                 |Anim| Notes
 ----+--------------------------------+-----+-----+-----+-----+------+------+
-shake_fist_anim("Standard Male Patient",     392) -- bloaty head patients lose head!
+shake_fist_anim("Standard Male Patient",   392) -- bloaty head patients lose head!
 
+assignPatientMarkers(shake_fist_animations, nil, {0.0, 0.0})
 
 --  | Vomit Animations                  |
 --  | Name                              |Anim | Notes
@@ -198,6 +250,8 @@ vomit_anim("Slack Male Patient",         4324)
 vomit_anim("Transparent Female Patient", 4452)
 vomit_anim("Transparent Male Patient",   4384)
 
+assignPatientMarkers(vomit_animations, nil, {0.0, 0.0})
+
 --  | Yawn Animations                  |
 --  | Name                              |Anim | Notes
 ----+-----------------------------------+-----+
@@ -206,6 +260,8 @@ yawn_anim("Standard Male Patient",      368)
 --yawn_anim("Alternate Male Patient",     2968)  is this one the same as standard male?
 -- whichever one is used for male, if he wears a hat it will lift when he yawns
 
+assignPatientMarkers(yawn_animations, nil, {0.0, 0.0})
+
 --  | Foot tapping Animations                  |
 --  | Name                              |Anim | Notes
 ----+-----------------------------------+-----+
@@ -213,6 +269,8 @@ tap_foot_anim("Standard Female Patient",    4464)
 tap_foot_anim("Standard Male Patient",      2960)
 tap_foot_anim("Alternate Male Patient",     360)
 
+assignPatientMarkers(tap_foot_animations, nil, {0.0, 0.0})
+
 --  | Check watch Animations                  |
 --  | Name                              |Anim | Notes
 ----+-----------------------------------+-----+
@@ -221,6 +279,8 @@ check_watch_anim("Standard Male Patient",      2964)
 check_watch_anim("Alternate Male Patient",     364)
 check_watch_anim("Slack Male Patient",         4060)
 
+assignPatientMarkers(check_watch_animations, nil, {0.0, 0.0})
+
 --  | pee Animations                  |
 --  | Name                              |Anim | Notes
 ----+-----------------------------------+-----+
@@ -235,53 +295,45 @@ pee_anim("Invisible Patient",          4208)
 pee_anim("Transparent Female Patient", 4852)
 pee_anim("Transparent Male Patient",   4848)
 
+assignPatientMarkers(pee_animations, nil, {0.0, 0.0})
+
+-- Some icons should only appear when the player hovers over the humanoid
+-- Higher priority is more important.
 --   | Available Moods |
---   | Name            |Icon|Priority|Show Always| Notes
+--   | Name            |Icon|Priority|On Hover| Notes
 -----+-----------------+----+--------+-----------+
-moods("reflexion",      4020,       5)            -- Some icons should only appear when the player
-moods("cantfind",       4050,       3)            -- hover over the humanoid
-moods("idea1",          2464,      10)            -- Higher priority is more important.
+moods("reflexion",      4020,       5)
+moods("cantfind",       4050,       3)
+moods("idea1",          2464,      10)
 moods("idea2",          2466,      11)
 moods("idea3",          4044,      12)
 moods("staff_wait",     4054,      20)
 moods("tired",          3990,      30)
 moods("pay_rise",       4576,      40)
 moods("thirsty",        3986,       4)
-moods("cold",           3994,       0,       true) -- These have no priority since
-moods("hot",            3988,       0,       true) -- they will be shown when hovering
-moods("queue",          4568,      70)             -- no matter what other priorities.
+moods("cold",           3994,       0,   true) -- These have no priority since
+moods("hot",            3988,       0,   true) -- they will be shown when hovering
+                                               -- no matter what other priorities.
+moods("queue",          4568,      70)
 moods("poo",            3996,       5)
 moods("sad_money",      4018,      55)
-moods("patient_wait",   5006,      40)
+moods("patient_wait",   5006,      39)
 moods("epidemy1",       4566,      55)
 moods("epidemy2",       4570,      55)
 moods("epidemy3",       4572,      55)
 moods("epidemy4",       4574,      55)
-moods("sad1",           3992,      40)
-moods("sad2",           4000,      41)
-moods("sad3",           4002,      42)
-moods("sad4",           4004,      43)
-moods("sad5",           4006,      44)
-moods("sad6",           4008,      45)
-moods("sad7",           4578,      46)
+moods("sad1",           3992,      39)          -- unused?
+moods("sad2",           4578,      40)
+moods("dying1",         4000,      41)
+moods("dying2",         4002,      42)
+moods("dying3",         4004,      43)
+moods("dying4",         4006,      44)
+moods("dying5",         4008,      45)
 moods("dead",           4046,      60)
 moods("cured",          4048,      60)
 moods("emergency",      3914,      50)
 moods("exit",           4052,      60)
 
-local anim_mgr = TheApp.animation_manager
-for anim in values(door_animations, "*.entering") do
-  anim_mgr:setMarker(anim, 0, {-1, 0}, 3, {-1, 0}, 9, {0, 0})
-end
-for anim in values(door_animations, "*.leaving") do
-  anim_mgr:setMarker(anim, 1, {0, 0.4}, 4, {0, 0.4}, 7, {0, 0}, 11, {0, -1})
-end
-for anim in values(door_animations, "*.entering_swing") do
-  anim_mgr:setMarker(anim, 0, {-1, 0}, 8, {0, 0})
-end
-for anim in values(door_animations, "*.leaving_swing") do
-  anim_mgr:setMarker(anim, 0, {0.1, 0}, 9, {0, -1})
-end
 
 --!param ... Arguments for base class constructor.
 function Humanoid:Humanoid(...)
@@ -341,6 +393,19 @@ function Humanoid:afterLoad(old, new)
   if old < 134 and new >= 134 then
     self.staff_change_callbacks = {}
   end
+  if old < 210 then
+    -- We renamed the old sad7 to sad2; and sad2 - sad6 to dying1 - dying5.
+    -- Make sure we adjust sad2 and sad7 to the new mood names
+    -- Other dying ones aren't an issue
+    if self:isMoodActive("sad2") then
+      self:setMood("sad2", "deactivate")
+      self:setMood("dying1", "activate")
+    end
+    if self:isMoodActive("sad7") then
+      self:setMood("sad7", "deactivate")
+      self:setMood("sad2", "activate")
+    end
+  end
 
   for _, action in pairs(self.action_queue) do
     -- Sometimes actions not actual instances of HumanoidAction
@@ -475,7 +540,9 @@ function Humanoid:startAction()
   -- Call the action start handler
   TheApp.humanoid_actions[action.name](action, self)
 
-  if action == self.action_queue[1] and action.todo_interrupt then
+  -- If action has been marked as to be interrupted
+  if action == self.action_queue[1] and action.todo_interrupt and not action.uninterruptible then
+    -- Interrupt this action
     local high_priority = action.todo_interrupt == "high"
     action.todo_interrupt = nil
     local on_interrupt = action.on_interrupt
@@ -538,14 +605,17 @@ function Humanoid:setNextAction(action, high_priority)
   -- Interrupt the current action and queue other actions to be interrupted
   -- when they start.
   if interrupted then
-    interrupted = queue[1]
+    local current_action = queue[1]
     for j = 1, i - 1 do
       queue[j].todo_interrupt = high_priority and "high" or true
     end
-    local on_interrupt = interrupted.on_interrupt
-    if on_interrupt then
-      interrupted.on_interrupt = nil
-      on_interrupt(interrupted, self, high_priority or false)
+    -- Try to interrupt current action
+    if not current_action.uninterruptible then
+      local interrupt_handler = current_action.on_interrupt
+      if interrupt_handler then
+        current_action.on_interrupt = nil
+        interrupt_handler(current_action, self, high_priority or false)
+      end
     end
   else
     -- Start the action if it has become the current action
@@ -822,11 +892,7 @@ end
 -- Unregisters a build callback for this humanoid.
 --!param callback (function) The callback to remove.
 function Humanoid:unregisterRoomBuildCallback(callback)
-  if self.build_callbacks[callback] then
-    self.build_callbacks[callback] = nil
-  else
-    self.world:gameLog("Warning: Trying to remove nonexistent room build callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
-  end
+  self.build_callbacks[callback] = nil
 end
 
 function Humanoid:notifyNewRoom(room)
@@ -858,8 +924,6 @@ function Humanoid:unregisterRoomRemoveCallback(callback)
   if self.remove_callbacks[callback] then
     self.world:unregisterRoomRemoveCallback(callback)
     self.remove_callbacks[callback] = nil
-  else
-    self.world:gameLog("Warning: Trying to remove nonexistent room remove callback (" .. tostring(callback) .. ") from humanoid (" .. tostring(self) .. ").")
   end
 end
 
