@@ -41,6 +41,10 @@ local multigesture_pinch_sensitivity_factor = 0.002
 -- will result in a call to adjustZoom in the onTick method
 local multigesture_pinch_amplification_factor = 100
 
+-- Speed of scrolling when using keys. Pixels / tick (18ms)
+-- This scroll speed is further adjusted by the configured scroll_speed
+local key_scroll_speed = 10
+
 --! Game UI constructor.
 --!param app (Application) Application object.
 --!param local_hospital Hospital to display
@@ -97,6 +101,7 @@ function GameUI:GameUI(app, local_hospital, map_editor)
   self.shake_screen_intensity = 0
 
   self.announcer = Announcer(app)
+  self.app:setCaptureMouse()
 end
 
 function GameUI:setupGlobalKeyHandlers()
@@ -104,10 +109,10 @@ function GameUI:setupGlobalKeyHandlers()
 
   -- Set the scrolling keys.
   self.scroll_keys = {
-     [tostring(self.app.hotkeys["ingame_scroll_up"])] = {x = 0, y = -10},
-     [tostring(self.app.hotkeys["ingame_scroll_down"])] = {x = 0, y = 10},
-     [tostring(self.app.hotkeys["ingame_scroll_left"])] = {x = -10, y = 0},
-     [tostring(self.app.hotkeys["ingame_scroll_right"])] = {x = 10, y = 0},
+     [tostring(self.app.hotkeys["ingame_scroll_up"])] = {x = 0, y = -key_scroll_speed},
+     [tostring(self.app.hotkeys["ingame_scroll_down"])] = {x = 0, y = key_scroll_speed},
+     [tostring(self.app.hotkeys["ingame_scroll_left"])] = {x = -key_scroll_speed, y = 0},
+     [tostring(self.app.hotkeys["ingame_scroll_right"])] = {x = key_scroll_speed, y = 0},
   }
 
   -- This is the long version of the shift speed key.
@@ -148,6 +153,7 @@ function GameUI:setupGlobalKeyHandlers()
   self:addKeyHandler("ingame_showmenubar", self, self.showMenuBar)
   self:addKeyHandler("ingame_gamespeed_speedup", self, self.keySpeedUp)
   self:addKeyHandler("ingame_setTransparent", self, self.keyTransparent)
+  self:addKeyHandler("ingame_toggleTransparent", self, self.toggleTransparent)
   self:addKeyHandler("ingame_toggleAdvisor", self, self.toggleAdviser)
   self:addKeyHandler("ingame_poopLog", self.app.world, self.app.world.dumpGameLog)
   self:addKeyHandler("ingame_poopStrings", self.app, self.app.dumpStrings)
@@ -163,7 +169,7 @@ function GameUI:setupGlobalKeyHandlers()
     self:addKeyHandler(string.format("ingame_recallPosition_%d", i), self, self.recallMapPosition, i)
   end
 
-  if self.app.config.debug then
+  if self.app.config.debug and self.app.world.map.level_number ~= "MAP EDITOR" then
     self:addKeyHandler("ingame_showCheatWindow", self, self.showCheatsWindow)
   end
 end
@@ -315,9 +321,9 @@ function GameUI:updateKeyScroll()
   if dx ~= 0 or dy ~= 0 then
     --Get the length of the scrolling vector.
     local mag = (dx^2 + dy^2) ^ 0.5
-    --Then normalize the scrolling vector, after which multiply it by the scroll speed variable used in self.scroll_keys, which is 10 as of 14/10/18.
-    dx = (dx / mag) * 10
-    dy = (dy / mag) * 10
+    --Then normalize the scrolling vector, after which multiply it by the scroll speed variable.
+    dx = (dx / mag) * key_scroll_speed
+    dy = (dy / mag) * key_scroll_speed
     -- Set the scroll amount to be used.
     self.tick_scroll_amount = {x = dx, y = dy}
     return true
@@ -336,6 +342,11 @@ function GameUI:keyTransparent()
   self:setWallsTransparent(true)
 end
 
+function GameUI:toggleTransparent()
+  self.toggled_transparency = not self.toggled_transparency
+  self:setWallsTransparent(self.toggled_transparency)
+end
+
 function GameUI:onKeyDown(rawchar, modifiers, is_repeat)
   if UI.onKeyDown(self, rawchar, modifiers, is_repeat) then
     -- Key has been handled already
@@ -377,7 +388,7 @@ function GameUI:onKeyUp(rawchar)
     self.app.world:previousSpeed()
   end
 
-  self:setWallsTransparent(false)
+  if not self.toggled_transparency then self:setWallsTransparent(false) end
 end
 
 function GameUI:makeDebugFax()
@@ -449,7 +460,7 @@ function GameUI:onCursorWorldPositionChange()
       local cursor = self.default_cursor
       if self.app.world.user_actions_allowed then
         --- If the patient is infected show the infected cursor
-        if epidemic and epidemic.coverup_in_progress and
+        if epidemic and epidemic.coverup_selected and
           entity and entity.infected and not epidemic.timer.closed then
           cursor = infected_cursor
           -- In vaccination mode display epidemic hover cursor for all entities
@@ -861,11 +872,12 @@ function GameUI:onTick()
     -- and defaulted to 2, where 1 was regular scroll speed. By
     -- By multiplying by 0.5, we allow for setting slower than normal
     -- scroll speeds, and ensure there is no behaviour change for players
-    -- who do not modify their config file.
+    -- who do not modify their config file. Later the tick speed was
+    -- doubled so now we multiply by 0.25.
     if self.shift_scroll_speed_pressed then
-      mult = mult * self.app.config.shift_scroll_speed * 0.5
+      mult = mult * self.app.config.shift_scroll_speed * 0.25
     else
-      mult = mult * self.app.config.scroll_speed * 0.5
+      mult = mult * self.app.config.scroll_speed * 0.25
     end
 
     self:scrollMap(dx * mult, dy * mult)
@@ -1285,6 +1297,7 @@ function GameUI:afterLoad(old, new)
 
   self.announcer.playing = false
 
+  self.app:setCaptureMouse()
   return UI.afterLoad(self, old, new)
 end
 
@@ -1306,6 +1319,8 @@ function GameUI:quit(mapeditor)
   local msg = mapeditor and _S.confirmation.quit_mapeditor or _S.confirmation.quit
   self:addWindow(UIConfirmDialog(self, false, msg, --[[persistable:gameui_confirm_quit]] function()
     self.app:loadMainMenu()
+    -- Release the mouse regardless of setting
+    self.app.video:setCaptureMouse(false)
   end))
 end
 
