@@ -1212,6 +1212,8 @@ confirmation = {
   music_warning = "Nota: Necesitas el archivo smpeg.dll o el equivalente para tu sistema operativo, de lo contrario no tendrás música en el juego. ¿Quieres continuar?",
   remove_destroyed_room = "¿Te gustaría eliminar la habitación por $%d?",
   replace_machine_extra_info = "La nueva máquina tendrá una resistencia de %d (actualmente es de %d).",
+  very_old_save = "Ha habido muchas actualizaciones en el juego desde que comenzaste. Para asegurarte de que todas las características funcionen como se espera, ¿te gustaría reiniciar este nivel ahora?//" ..
+  "Tu archivo de guardado anterior no se eliminará a menos que lo sobrescribas."
 }
 
 information = {
@@ -1225,9 +1227,9 @@ information = {
     reputation = "Tu reputación ha caído por debajo de %d.",
     balance = "Tu cuenta bancaria ha llegado a tener menos de %d.",
     percentage_killed = "Has matado a más de un %d por ciento de los pacientes.",
-    cheat = "¿Lo has elegido tú o te has equivocado de botón? No sabes ni hacer trampas como es debido... ¿a que no tiene gracia?",
+    cheat = "¡Espero que no hayas hecho clic en el botón 'Perder Nivel' por accidente!",
   },
-  cheat_not_possible = "No puedes usar ese truco en este nivel. No sabes ni hacer trampas, ¿a que no tiene gracia?",
+  cheat_not_possible = "No puedes usar ese truco en este nivel.",
 }
 
 tooltip.information = {
@@ -1392,6 +1394,42 @@ map_editor_window = {
   }
 }
 
+machine_menu = {
+  machine = "Máquina",
+  remaining_strength = "Restante",
+  total_strength = "Fuerza",
+  ratio = "Ratio",
+  close = "Cerrar",
+}
+
+tooltip.machine_menu = {
+  sort = "Haz clic para ordenar por esta valor.",
+  machine = "Lista de máquinas - haz clic en una máquina para abrir su interfaz y desplazarte a su ubicación",
+  smoking = "Esta casilla está marcada si la máquina corre riesgo de explotar",
+  assigned = "Esta casilla está marcada si un operario de mantenimiento está asignado para reparar la máquina correspondiente",
+  remaining_strength = "Esta etiqueta muestra la fuerza restante de la máquina",
+  total_strength = "Muestra la fuerza total de la máquina",
+  ratio = "Muestra la relación entre la fuerza restante y la fuerza total",
+  header = {
+    smoking = "Indicador de peligro",
+    assigned = "Indicador de asignación",
+    machine = "Nombre y posición de la máquina",
+    remaining_strength = "Fuerza restante de las máquinas.",
+    total_strength = "Fuerza total de las máquinas.",
+    ratio = "Porcentaje de fuerza restante en relación con la fuerza total de las máquinas",
+    times_used = "Cantidad de veces que se ha usado la máquina.",
+  },
+  close = "Cerrar la lista de máquinas",
+}
+
+menu_charts = {
+  machine_menu  = "  (%1%) MENÚ DE MÁQUINAS",
+}
+
+hotkey_window = {
+  ingame_panel_machineMenu = "Menú de Máquinas",
+}
+
 hotkeys_file_err = {
   file_err_01 = "No se pudo cargar el archivo 'hotkeys.txt'. Por favor asegúrate de que CorsixTH " ..
         "tiene permisos para leer/escribir ",
