@@ -95,17 +95,6 @@ local function wait_for_object(humanoid, obj, must_happen)
   return IdleAction():setMustHappen(must_happen):setLoopCallback(loop_callback_wait)
 end
 
---! Returns true if an operation is ongoing
-function OperatingTheatreRoom:isOperating()
-  for k, _ in pairs(self.staff_member_set) do
-    if k:getCurrentAction().name == "multi_use_object" then
-      return true
-    end
-  end
-
-  return false
-end
-
 --! Builds the second operation action (i.e. with the surgeon whose we
 --! see the back). Called either when the operation starts or when the
 --! operation is resumed after interruption caused by the picking up of
@@ -128,8 +117,12 @@ function OperatingTheatreRoom._buildTableAction2(multi_use, operation_table_b)
     multi_use.prolonged_usage = false
   end
 
-  return UseObjectAction(operation_table_b):setLoopCallback(loop_callback_use_object)
-      :setAfterUse(after_use_use_object):setMustHappen(true):disableTruncate()
+  return UseObjectAction(operation_table_b)
+    :setLoopCallback(loop_callback_use_object)
+    :setAfterUse(after_use_use_object)
+    :setMustHappen(true)
+    :disableTruncate()
+    :setUninterruptable(true)
 end
 
 function OperatingTheatreRoom:commandEnteringStaff(staff)
@@ -139,18 +132,6 @@ function OperatingTheatreRoom:commandEnteringStaff(staff)
   staff:queueAction(wait_for_object(staff, screen, false))
   staff:queueAction(UseScreenAction(screen))
 
-  -- Resume operation if already ongoing
-  if self:isOperating() then
-    local surgeon1 = next(self.staff_member_set)
-    local ongoing_action = surgeon1:getCurrentAction()
-    assert(ongoing_action.name == "multi_use_object")
-
-    local table, table_x, table_y = self.world:findObjectNear(staff, "operating_table_b")
-    self:queueWashHands(staff)
-    staff:queueAction(WalkAction(table_x, table_y))
-    staff:queueAction(self._buildTableAction2(ongoing_action, table))
-  end
-
   self.staff_member_set[staff] = true
 
   -- Wait around for patients
@@ -202,17 +183,25 @@ end
 
 --! Sends the surgeon to the nearest operation sink ("op_sink1")
 --! and makes him wash his hands
---!param at_front (boolean): If true, add the actions at the front the action queue.
+--!param before_other_actions (boolean): If true, add the actions at the front the action queue.
 --! Add the actions at the end of the queue otherwise.
 --! Default value is true.
-function OperatingTheatreRoom:queueWashHands(surgeon, at_front)
+function OperatingTheatreRoom:queueWashHands(surgeon, before_other_actions)
   local sink, sink_x, sink_y = self.world:findObjectNear(surgeon, "op_sink1")
-  local walk = WalkAction(sink_x, sink_y):setMustHappen(true):disableTruncate()
+  local walk = WalkAction(sink_x, sink_y)
+    :setMustHappen(true)
+    :disableTruncate()
   local wait = wait_for_object(surgeon, sink, true)
-  local wash = UseObjectAction(sink):setMustHappen(true)
+    :setMustHappen(true)
+    :disableTruncate()
+    :setUninterruptable(true)
+  local wash = UseObjectAction(sink)
+    :setMustHappen(true)
+    :disableTruncate()
+    :setUninterruptable(true)
 
   for pos, action in pairs({walk, wait, wash}) do
-    if (at_front) then
+    if (before_other_actions) then
       surgeon:queueAction(action, pos)
     else
       surgeon:queueAction(action)
@@ -237,10 +226,15 @@ function OperatingTheatreRoom:commandEnteringPatient(patient)
   local surgeon2 = next(self.staff_member_set, surgeon1)
   assert(surgeon1 and surgeon2, "Not enough staff in operating theatre")
 
-  -- Patient changes into surgical gown
   local screen, sx, sy = self.world:findObjectNear(patient, "surgeon_screen")
-  patient:walkTo(sx, sy)
-  patient:queueAction(UseScreenAction(screen))
+  -- Patient walk to surgeon screen
+  patient:queueAction(WalkAction(sx, sy)
+    :setMustHappen(true)
+    :disableTruncate())
+  -- Patient changes into surgical gown
+  patient:queueAction(UseScreenAction(screen)
+    :setMustHappen(true)
+    :disableTruncate())
 
   -- Meanwhile, surgeons wash their hands
   -- TODO: They sometimes overlap each other when doing that. Can we avoid that?
