@@ -256,7 +256,7 @@ end
 --!  can be either of "left", "center", "right"
 function Panel:setLabel(label, font, align)
   self.label = label or ""
-  self.label_font = font or self.label_font or TheApp.gfx:loadFont("QData", "Font01V")
+  self.label_font = font or self.label_font or TheApp.gfx:loadFontAndSpriteTable("QData", "Font01V")
   self.align = align or self.align
   return self
 end
@@ -1538,9 +1538,6 @@ top-left corner of the window.
 top-left corner of the window.
 ]]
 function Window:hitTest(x, y)
-  if x < 0 or y < 0 or (self.width and x >= self.width) or (self.height and y >= self.height) then
---    return false
-  end
   if self.panels[1] then
     for _, panel in ipairs(self.panels) do
       if self:hitTestPanel(x, y, panel) then
@@ -1712,8 +1709,8 @@ local --[[persistable:window_drag_position_representation]] function getNicestPo
 end
 
 --[[ Initiate dragging of the window.
-!param x The X position of the cursor in window co-ordinatees.
-!param y The Y position of the cursor in window co-ordinatees.
+!param x The X position of the cursor in window coordinates.
+!param y The Y position of the cursor in window coordinates.
 ]]
 function Window:beginDrag(x, y)
   if not self.width or not self.height or not self.ui or
