@@ -1,7 +1,7 @@
 #ifndef CORSIX_TH_LUA_RNC
 #define CORSIX_TH_LUA_RNC
 
-#include "th_lua.h"
+#include "lua.hpp"
 
 int luaopen_rnc(lua_State* L);
 
