@@ -22,16 +22,19 @@ SOFTWARE.
 
 #include "persist_lua.h"
 
+#include "config.h"
+
 #include <errno.h>
 
 #include <array>
+#include <climits>
 #include <cmath>
 #include <cstdio>
-#include <cstdlib>
 #include <cstring>
+#include <new>
 #include <string>
-#include <vector>
 
+#include "lua.hpp"
 #include "th_lua.h"
 #ifdef _MSC_VER
 #pragma warning( \
@@ -1158,7 +1161,7 @@ int l_persist_dofile(lua_State* L) {
   }
   size_t iBufferSize = lua_objlen(L, luaT_upvalueindex(1));
   size_t iBufferUsed = 0;
-  while (!std::feof(fFile)) {
+  while (!std::ferror(fFile) && !std::feof(fFile)) {
     iBufferUsed += std::fread(
         reinterpret_cast<char*>(lua_touserdata(L, luaT_upvalueindex(1))) +
             iBufferUsed,
