@@ -20,9 +20,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */
 
-#include "config.h"
-
-#include "lua_sdl.h"
+#include "lua.hpp"
 #include "th_lua.h"
 #ifdef CORSIX_TH_USE_WIN32_SDK
 #include <SDL_syswm.h>
@@ -30,6 +28,8 @@ SOFTWARE.
 
 #include "../resource.h"
 #endif
+#include <SDL_mouse.h>
+
 #include <array>
 
 namespace {
