@@ -27,10 +27,14 @@ SOFTWARE.
 #include <algorithm>
 #include <cassert>
 #include <climits>
+#include <cstdlib>
 #include <cstring>
+#include <memory>
 #include <new>
 
 #include "persist_lua.h"
+#include "th_gfx_sdl.h"
+#include "th_lua.h"
 #include "th_map.h"
 #include "th_sound.h"
 
@@ -227,8 +231,8 @@ bool animation_manager::load_from_th_file(
     oFrame.sound = pFrame->sound;
     oFrame.flags = pFrame->flags;
     // Bounding box fields initialised later
-    oFrame.marker_x = 0;
-    oFrame.marker_y = 0;
+    oFrame.primary_marker_x = 0;
+    oFrame.primary_marker_y = 0;
     oFrame.secondary_marker_x = 0;
     oFrame.secondary_marker_y = 0;
 
@@ -638,8 +642,8 @@ bool animation_manager::load_custom_animations(const uint8_t* pData,
 
       // Set later
       oFrame.flags = 0;
-      oFrame.marker_x = 0;
-      oFrame.marker_y = 0;
+      oFrame.primary_marker_x = 0;
+      oFrame.primary_marker_y = 0;
       oFrame.secondary_marker_x = 0;
       oFrame.secondary_marker_y = 0;
 
@@ -784,13 +788,14 @@ void animation_manager::set_animation_alt_palette_map(size_t iAnimation,
   } while (iFrame != iFirstFrame);
 }
 
-bool animation_manager::set_frame_marker(size_t iFrame, int iX, int iY) {
+bool animation_manager::set_frame_primary_marker(size_t iFrame, int iX,
+                                                 int iY) {
   if (iFrame >= frame_count) {
     return false;
   }
 
-  frames[iFrame].marker_x = iX;
-  frames[iFrame].marker_y = iY;
+  frames[iFrame].primary_marker_x = iX;
+  frames[iFrame].primary_marker_y = iY;
   return true;
 }
 
@@ -805,13 +810,14 @@ bool animation_manager::set_frame_secondary_marker(size_t iFrame, int iX,
   return true;
 }
 
-bool animation_manager::get_frame_marker(size_t iFrame, int* pX, int* pY) {
+bool animation_manager::get_frame_primary_marker(size_t iFrame, int* pX,
+                                                 int* pY) {
   if (iFrame >= frame_count) {
     return false;
   }
 
-  *pX = frames[iFrame].marker_x;
-  *pY = frames[iFrame].marker_y;
+  *pX = frames[iFrame].primary_marker_x;
+  *pY = frames[iFrame].primary_marker_y;
   return true;
 }
 
@@ -1166,11 +1172,16 @@ void animation::draw(render_target* pCanvas, int iDestX, int iDestY) {
   }
 }
 
-void animation::draw_child(render_target* pCanvas, int iDestX, int iDestY) {
+void animation::draw_child(render_target* pCanvas, int iDestX, int iDestY,
+                           bool use_primary) {
   if (are_flags_set(flags, thdf_alpha_50 | thdf_alpha_75)) return;
   if (are_flags_set(parent->flags, thdf_alpha_50 | thdf_alpha_75)) return;
   int iX = 0, iY = 0;
-  parent->get_marker(&iX, &iY);
+  if (use_primary)
+    parent->get_primary_marker(&iX, &iY);
+  else
+    parent->get_secondary_marker(&iX, &iY);
+
   iX += x_relative_to_tile + iDestX;
   iY += y_relative_to_tile + iDestY;
   if (sound_to_play) {
@@ -1249,55 +1260,6 @@ bool animation::hit_test_morph(int iDestX, int iDestY, int iTestX, int iTestY) {
          morph_target->hit_test(iDestX, iDestY, iTestX, iTestY);
 }
 
-namespace {
-bool THAnimation_hit_test_child(drawable* pSelf, int iDestX, int iDestY,
-                                int iTestX, int iTestY) {
-  return reinterpret_cast<animation*>(pSelf)->hit_test_child(iDestX, iDestY,
-                                                             iTestX, iTestY);
-}
-
-void THAnimation_draw_child(drawable* pSelf, render_target* pCanvas, int iDestX,
-                            int iDestY) {
-  reinterpret_cast<animation*>(pSelf)->draw_child(pCanvas, iDestX, iDestY);
-}
-
-bool THAnimation_hit_test_morph(drawable* pSelf, int iDestX, int iDestY,
-                                int iTestX, int iTestY) {
-  return reinterpret_cast<animation*>(pSelf)->hit_test_morph(iDestX, iDestY,
-                                                             iTestX, iTestY);
-}
-
-void THAnimation_draw_morph(drawable* pSelf, render_target* pCanvas, int iDestX,
-                            int iDestY) {
-  reinterpret_cast<animation*>(pSelf)->draw_morph(pCanvas, iDestX, iDestY);
-}
-
-bool THAnimation_hit_test(drawable* pSelf, int iDestX, int iDestY, int iTestX,
-                          int iTestY) {
-  return reinterpret_cast<animation*>(pSelf)->hit_test(iDestX, iDestY, iTestX,
-                                                       iTestY);
-}
-
-void THAnimation_draw(drawable* pSelf, render_target* pCanvas, int iDestX,
-                      int iDestY) {
-  reinterpret_cast<animation*>(pSelf)->draw(pCanvas, iDestX, iDestY);
-}
-
-bool THAnimation_is_multiple_frame_animation(drawable* pSelf) {
-  animation* pAnimation = reinterpret_cast<animation*>(pSelf);
-  if (pAnimation) {
-    size_t firstFrame = pAnimation->get_animation_manager()->get_first_frame(
-        pAnimation->get_animation());
-    size_t nextFrame =
-        pAnimation->get_animation_manager()->get_next_frame(firstFrame);
-    return nextFrame != firstFrame;
-  } else {
-    return false;
-  }
-}
-
-}  // namespace
-
 animation_base::animation_base() : drawable() {
   x_relative_to_tile = 0;
   y_relative_to_tile = 0;
@@ -1315,10 +1277,8 @@ animation::animation()
       frame_index(0),
       speed({0, 0}),
       sound_to_play(0),
-      crop_column(0) {
-  draw_fn = THAnimation_draw;
-  hit_test_fn = THAnimation_hit_test;
-  is_multiple_frame_animation_fn = THAnimation_is_multiple_frame_animation;
+      crop_column(0),
+      anim_kind(animation_kind::normal) {
   patient_effect = animation_effect::none;
   patient_effect_offset = rand();
 }
@@ -1328,7 +1288,11 @@ void animation::persist(lua_persist_writer* pWriter) const {
 
   // Write the next chained thing
   lua_rawgeti(L, luaT_environindex, 2);
-  lua_pushlightuserdata(L, next);
+  void* np = dynamic_cast<animation*>(next);
+  if (np == nullptr) {
+    np = dynamic_cast<sprite_render_list*>(next);
+  }
+  lua_pushlightuserdata(L, np);
   lua_rawget(L, -2);
   pWriter->fast_write_stack_object(-1);
   lua_pop(L, 2);
@@ -1336,13 +1300,11 @@ void animation::persist(lua_persist_writer* pWriter) const {
   // Write the drawable fields
   pWriter->write_uint(flags);
 
-  if (draw_fn == THAnimation_draw && hit_test_fn == THAnimation_hit_test) {
+  if (anim_kind == animation_kind::normal) {
     pWriter->write_uint(1);
-  } else if (draw_fn == THAnimation_draw_child &&
-             hit_test_fn == THAnimation_hit_test_child) {
+  } else if (anim_kind == animation_kind::primary_child) {
     pWriter->write_uint(2);
-  } else if (draw_fn == THAnimation_draw_morph &&
-             hit_test_fn == THAnimation_hit_test_morph) {
+  } else if (anim_kind == animation_kind::morph) {
     // NB: Prior version of code used the number 3 here, and forgot
     // to persist the morph target.
     pWriter->write_uint(4);
@@ -1351,6 +1313,8 @@ void animation::persist(lua_persist_writer* pWriter) const {
     lua_rawget(L, -2);
     pWriter->write_stack_object(-1);
     lua_pop(L, 2);
+  } else if (anim_kind == animation_kind::secondary_child) {
+    pWriter->write_uint(5);
   } else {
     pWriter->write_uint(0);
   }
@@ -1371,7 +1335,8 @@ void animation::persist(lua_persist_writer* pWriter) const {
   }
 
   // Write the unioned fields
-  if (draw_fn != THAnimation_draw_child) {
+  if (anim_kind != animation_kind::primary_child &&
+      anim_kind != animation_kind::secondary_child) {
     pWriter->write_int(speed.dx);
     pWriter->write_int(speed.dy);
   } else {
@@ -1397,7 +1362,8 @@ void animation::depersist(lua_persist_reader* pReader) {
   do {
     // Read the chain
     if (!pReader->read_stack_object()) break;
-    next = reinterpret_cast<link_list*>(lua_touserdata(L, -1));
+
+    next = luaT_toanimationbase(L, -1);
     if (next) next->prev = this;
     lua_pop(L, 1);
 
@@ -1411,20 +1377,20 @@ void animation::depersist(lua_persist_reader* pReader) {
         // missing, so settle for a graphical bug rather than a segfault
         // by reverting to the normal function set.
       case 1:
-        draw_fn = THAnimation_draw;
-        hit_test_fn = THAnimation_hit_test;
+        set_animation_kind(animation_kind::normal);
         break;
       case 2:
-        draw_fn = THAnimation_draw_child;
-        hit_test_fn = THAnimation_hit_test_child;
+        set_animation_kind(animation_kind::primary_child);
         break;
       case 4:
-        draw_fn = THAnimation_draw_morph;
-        hit_test_fn = THAnimation_hit_test_morph;
+        set_animation_kind(animation_kind::morph);
         pReader->read_stack_object();
         morph_target = reinterpret_cast<animation*>(lua_touserdata(L, -1));
         lua_pop(L, 1);
         break;
+      case 5:
+        set_animation_kind(animation_kind::secondary_child);
+        break;
       default:
         pReader->set_error(lua_pushfstring(
             L, "Unknown animation function set #%i", iFunctionSet));
@@ -1450,7 +1416,8 @@ void animation::depersist(lua_persist_reader* pReader) {
     }
 
     // Read the unioned fields
-    if (draw_fn != THAnimation_draw_child) {
+    if (anim_kind != animation_kind::primary_child &&
+        anim_kind != animation_kind::secondary_child) {
       if (!pReader->read_int(speed.dx)) break;
       if (!pReader->read_int(speed.dy)) break;
     } else {
@@ -1494,29 +1461,75 @@ void animation::set_patient_effect(animation_effect patient_effect) {
   this->patient_effect = patient_effect;
 }
 
+void animation::set_animation_kind(animation_kind anim_kind) {
+  this->anim_kind = anim_kind;
+}
+
 namespace {
+//! Storage of one or two sounds. If two sounds are stored, both are selected in
+//! 50% of the cases.
+struct sound_pair {
+  sound_pair(int16_t sound) : soundA(sound), soundB(-1) {}
+  sound_pair(int16_t soundA, int16_t soundB) : soundA(soundA), soundB(soundB) {}
+
+  int16_t get_sound() const {
+    if (soundB < 0) {
+      return soundA;
+    } else {
+      int value = rand();
+      int counter = 0;
+      for (int i = 0; i < 7; i++) {
+        counter = counter + (value & 1);
+        value >>= 1;
+      }
+      return (counter < 4) ? soundA : soundB;
+    }
+  }
+
+ private:
+  const int16_t soundA;  //!< First available sound.
+  const int16_t soundB;  //!< If non-negative, the second available sound.
+};
+
+typedef std::map<size_t, sound_pair> sound_replacement_map;
+
 // Map of frame numbers to sounds to play.
-const std::map<size_t, int> frame_sound_replacements{
+const sound_replacement_map frame_sound_replacements{
     // Female flying to heaven (anim 3220)
-    {6987, 123},
+    {6987, sound_pair(123)},
+
     // Using Computer (anim 2098)
-    {4213, 35},
-    {4215, 35},
-    {4224, 35},
-    {4230, 35},
+    {4213, sound_pair(35)},
+    {4215, sound_pair(35)},
+    {4224, sound_pair(35)},
+    {4230, sound_pair(35)},
+
+    // Vomit sounds.
+    {1902, sound_pair(58, 114)},   // Animation 1034
+    {4149, sound_pair(58, 114)},   // Animation 2056
+    {6901, sound_pair(58, 114)},   // Animation 3184
+    {8819, sound_pair(58, 114)},   // Animation 4138
+    {9105, sound_pair(58, 114)},   // Animation 4204
+    {9565, sound_pair(58, 114)},   // Animation 4324
+    {9654, sound_pair(58, 114)},   // Animation 4384
+    {9944, sound_pair(58, 114)},   // Animation 4452
+    {10007, sound_pair(58, 114)},  // Animation 4476
+    {10989, sound_pair(58, 114)},  // Animation 4792
+
     // Using Atom Analyser. Actually a generic animation of Researcher pushing
     // buttons (anim 4878)
-    {11136, 35},
-    {11138, 35},
-    {11147, 35},
-    {11152, 35},
-    {11153, 35},
-    {11154, 35}};
+    {11136, sound_pair(35)},
+    {11138, sound_pair(35)},
+    {11147, sound_pair(35)},
+    {11152, sound_pair(35)},
+    {11153, sound_pair(35)},
+    {11154, sound_pair(35)}};
 }  // Namespace
 
 void animation::tick() {
   frame_index = manager->get_next_frame(frame_index);
-  if (draw_fn != THAnimation_draw_child) {
+  if (anim_kind != animation_kind::primary_child &&
+      anim_kind != animation_kind::secondary_child) {
     x_relative_to_tile += speed.dx;
     y_relative_to_tile += speed.dy;
   }
@@ -1529,12 +1542,12 @@ void animation::tick() {
   }
 
   // Decide sound to play.
-  std::map<size_t, int>::const_iterator pos =
+  sound_replacement_map::const_iterator pos =
       frame_sound_replacements.find(frame_index);
   if (pos == frame_sound_replacements.end()) {
     sound_to_play = manager->get_frame_sound(frame_index);
   } else {
-    sound_to_play = pos->second;
+    sound_to_play = pos->second.get_sound();
   }
 }
 
@@ -1566,15 +1579,14 @@ void animation_base::attach_to_tile(map_tile* pMapNode, int layer) {
   pList->next = this;
 }
 
-void animation::set_parent(animation* pParent) {
+void animation::set_parent(animation* pParent, bool use_primary) {
   remove_from_tile();
   if (pParent == nullptr) {
-    draw_fn = THAnimation_draw;
-    hit_test_fn = THAnimation_hit_test;
+    set_animation_kind(animation_kind::normal);
     speed = {0, 0};
   } else {
-    draw_fn = THAnimation_draw_child;
-    hit_test_fn = THAnimation_hit_test_child;
+    set_animation_kind(use_primary ? animation_kind::primary_child
+                                   : animation_kind::secondary_child);
     parent = pParent;
     next = parent->next;
     if (next) next->prev = this;
@@ -1589,13 +1601,12 @@ void animation::set_animation(animation_manager* pManager, size_t iAnimation) {
   frame_index = pManager->get_first_frame(iAnimation);
   if (morph_target) {
     morph_target = nullptr;
-    draw_fn = THAnimation_draw;
-    hit_test_fn = THAnimation_hit_test;
+    set_animation_kind(animation_kind::normal);
   }
 }
 
-bool animation::get_marker(int* pX, int* pY) {
-  if (!manager || !manager->get_frame_marker(frame_index, pX, pY)) {
+bool animation::get_primary_marker(int* pX, int* pY) {
+  if (!manager || !manager->get_frame_primary_marker(frame_index, pX, pY)) {
     return false;
   }
 
@@ -1654,8 +1665,7 @@ int GetAnimationDurationAndExtent(animation_manager* pManager, size_t iFrame,
 
 void animation::set_morph_target(animation* pMorphTarget, int iDurationFactor) {
   morph_target = pMorphTarget;
-  draw_fn = THAnimation_draw_morph;
-  hit_test_fn = THAnimation_hit_test_morph;
+  set_animation_kind(animation_kind::morph);
 
   /* Morphing is the process by which two animations are combined to give a
   single animation of one animation turning into another. At the moment,
@@ -1712,31 +1722,7 @@ void animation_base::set_layer(int iLayer, int iId) {
   }
 }
 
-namespace {
-
-bool THSpriteRenderList_hit_test(drawable* pSelf, int iDestX, int iDestY,
-                                 int iTestX, int iTestY) {
-  return reinterpret_cast<sprite_render_list*>(pSelf)->hit_test(iDestX, iDestY,
-                                                                iTestX, iTestY);
-}
-
-void THSpriteRenderList_draw(drawable* pSelf, render_target* pCanvas,
-                             int iDestX, int iDestY) {
-  reinterpret_cast<sprite_render_list*>(pSelf)->draw(pCanvas, iDestX, iDestY);
-}
-
-bool THSpriteRenderList_is_multiple_frame_animation(drawable* pSelf) {
-  return false;
-}
-
-}  // namespace
-
-sprite_render_list::sprite_render_list() : animation_base() {
-  draw_fn = THSpriteRenderList_draw;
-  hit_test_fn = THSpriteRenderList_hit_test;
-  is_multiple_frame_animation_fn =
-      THSpriteRenderList_is_multiple_frame_animation;
-}
+sprite_render_list::sprite_render_list() : animation_base() {}
 
 sprite_render_list::~sprite_render_list() { delete[] sprites; }
 
@@ -1852,7 +1838,11 @@ void sprite_render_list::persist(lua_persist_writer* pWriter) const {
 
   // Write the next chained thing
   lua_rawgeti(L, luaT_environindex, 2);
-  lua_pushlightuserdata(L, next);
+  void* np = dynamic_cast<animation*>(next);
+  if (np == nullptr) {
+    np = dynamic_cast<sprite_render_list*>(next);
+  }
+  lua_pushlightuserdata(L, np);
   lua_rawget(L, -2);
   pWriter->fast_write_stack_object(-1);
   lua_pop(L, 2);
@@ -1907,7 +1897,7 @@ void sprite_render_list::depersist(lua_persist_reader* pReader) {
     return;
   }
 
-  next = reinterpret_cast<link_list*>(lua_touserdata(L, -1));
+  next = luaT_toanimationbase(L, -1);
   if (next) {
     next->prev = this;
   }
