@@ -26,19 +26,23 @@ SOFTWARE.
 #ifdef CORSIX_TH_USE_FREETYPE2
 #include "th_gfx_font.h"
 #endif
+#include <SDL.h>
 
 #include <algorithm>
 #include <cmath>
 #include <cstdio>
-#include <cstdlib>
 #include <cstring>
 #include <iostream>
 #include <limits>
 #include <memory>
 #include <new>
+#include <stack>
 #include <stdexcept>
+#include <vector>
 
-#include "th_map.h"
+#include "persist_lua.h"
+#include "th_gfx_common.h"
+#include "th_gfx_sdl.h"
 
 #if SDL_VERSION_ATLEAST(2, 0, 10)
 
@@ -556,7 +560,7 @@ bool render_target::set_scale_factor(double fScale, scaled_items eWhatToScale) {
     if (!zoom_buffer->is_target()) {
       global_scale_factor = 1.0;
       std::cout << "Warning: Could not render to zoom texture - "
-                << SDL_GetError() << std::endl;
+                << SDL_GetError() << "\n";
 
       return false;
     }
@@ -589,7 +593,7 @@ const char* render_target::get_renderer_details() const {
 const char* render_target::get_last_error() { return SDL_GetError(); }
 
 bool render_target::start_frame() {
-  // Destroy any intermediate texures used last frame.
+  // Destroy any intermediate textures used last frame.
   destroy_intermediate_textures();
 
   fill_black();
@@ -858,8 +862,11 @@ void render_target::draw(SDL_Texture* pTexture, const SDL_Rect* prcSrcRect,
   getScaleRect(prcDstRect, draw_scale(), &scaledDstRect);
   if (current_target) current_target->offset(scaledDstRect);
   if (iSDLFlip != 0) {
+    // iSDLFlip may be 3 (HORIZONTAL | VERTICAL) but there is no enum value for
+    // that
     SDL_RenderCopyExF(renderer, pTexture, prcSrcRect, &scaledDstRect, 0,
-                      nullptr, (SDL_RendererFlip)iSDLFlip);
+                      nullptr,
+                      (SDL_RendererFlip)iSDLFlip);  // NOLINT
   } else {
     SDL_RenderCopyF(renderer, pTexture, prcSrcRect, &scaledDstRect);
   }
@@ -1204,12 +1211,24 @@ bool sprite_sheet::get_sprite_average_colour(size_t iSprite,
     uint8_t cPalIndex = pSprite->data[i];
     uint32_t iColour = palette->get_argb_data()[cPalIndex];
     if ((iColour >> 24) == 0) continue;
+
+    // - Convert from nonlinear RGB to linear RGB.
+    //   gamma = around 2.2 but depends on the screen.
+    //   R, G and B are fractions.
+    const double gamma = 2.2;
+    double linR = std::pow(palette::get_red(iColour) / 255.0, gamma);
+    double linG = std::pow(palette::get_green(iColour) / 255.0, gamma);
+    double linB = std::pow(palette::get_blue(iColour) / 255.0, gamma);
+    // - Compute luminance Y in XYZ space.
+    //   Y = .2126 * R^gamma + .7152 * G^gamma + .0722 * B^gamma
+    double Y = 0.2126 * linR + 0.7152 * linG + 0.0722 * linB;
+    // - Compute lightness L*.
+    //   L* = 116 * Y ^ 1/3 - 16, range is from 0 to 100.
+    double L = std::min(100.0, std::max(0.0, 116 * std::cbrt(Y) - 16));
+    uint8_t cIntensity = static_cast<uint8_t>(L / 100 * 255.0);
+
     // Grant higher score to pixels with high or low intensity (helps avoid
     // grey fonts)
-    int iR = palette::get_red(iColour);
-    int iG = palette::get_green(iColour);
-    int iB = palette::get_blue(iColour);
-    uint8_t cIntensity = static_cast<uint8_t>((iR + iG + iB) / 3);
     int iScore = 1 + std::max(0, 3 - ((255 - cIntensity) / 32)) +
                  std::max(0, 3 - (cIntensity / 32));
     iUsageCounts[cPalIndex] += iScore;
@@ -1225,6 +1244,18 @@ bool sprite_sheet::get_sprite_average_colour(size_t iSprite,
   return true;
 }
 
+bool sprite_sheet::is_sprite_visible(size_t iSprite) const {
+  if (iSprite >= sprite_count) return false;
+
+  const sprite* pSprite = sprites + iSprite;
+  for (long i = 0; i < pSprite->width * pSprite->height; ++i) {
+    uint8_t cPalIndex = pSprite->data[i];
+    uint32_t iColour = palette->get_argb_data()[cPalIndex];
+    if ((iColour >> 24) != 0) return true;
+  }
+  return false;
+}
+
 void sprite_sheet::draw_sprite(render_target* pCanvas, size_t iSprite, int iX,
                                int iY, uint32_t iFlags, size_t effect_ticks,
                                animation_effect effect) {
@@ -1623,7 +1654,7 @@ void freetype_font::make_texture(render_target* pEventualCanvas,
               pCacheEntry->width * pCacheEntry->height * sizeof(uint32_t));
   uint8_t* pInRow = pCacheEntry->data;
   uint32_t* pOutRow = pPixels;
-  uint32_t iColBase = colour & 0xFFFFFF;
+  uint32_t iColBase = font_colour & 0xFFFFFF;
   for (int iY = 0; iY < pCacheEntry->height;
        ++iY, pOutRow += pCacheEntry->width, pInRow += pCacheEntry->width) {
     for (int iX = 0; iX < pCacheEntry->width; ++iX) {
