@@ -32,11 +32,12 @@ SOFTWARE.
 #include <stdexcept>
 #include <vector>
 
-#include "persist_lua.h"
-#include "th.h"
 #include "th_gfx_common.h"
 
 class cursor;
+class line_sequence;
+class lua_persist_reader;
+class lua_persist_writer;
 
 struct clip_rect : public SDL_Rect {
   typedef Sint16 x_y_type;
@@ -562,6 +563,13 @@ class sprite_sheet {
   */
   bool get_sprite_average_colour(size_t iSprite, argb_colour* pColour) const;
 
+  //! Return whether the given sprite show any pixel when displayed.
+  /*!
+      @param iSprite Sprite number to analyze.
+      @return Whether any pixel of the sprite is visible.
+  */
+  bool is_sprite_visible(size_t iSprite) const;
+
   //! Draw a sprite onto the canvas.
   /*!
       @param pCanvas Canvas to draw on.
@@ -640,7 +648,7 @@ class sprite_sheet {
 
     //! Height of the sprite.
     int height;
-  } * sprites;
+  }* sprites;
 
   //! Original palette.
   const ::palette* palette;
