@@ -24,19 +24,21 @@ SOFTWARE.
 
 #include "config.h"
 
-#include <SDL.h>
-
 #include <algorithm>
 #include <cstdio>
 #include <cstring>
-#include <exception>
 #include <fstream>
 #include <new>
+#include <stdexcept>
 #include <string>
 
+#include "lua.hpp"
+#include "persist_lua.h"
 #include "run_length_encoder.h"
 #include "th.h"
 #include "th_gfx.h"
+#include "th_gfx_sdl.h"
+#include "th_lua.h"
 #include "th_map_overlays.h"
 
 constexpr int max_player_count = 4;
@@ -246,18 +248,18 @@ map_tile_flags::operator uint32_t() const {
   return raw;
 }
 
-map_tile::map_tile() : iParcelId(0), iRoomId(0), flags({}), objects() {
-  iBlock[0] = 0;
-  iBlock[1] = 0;
-  iBlock[2] = 0;
-  iBlock[3] = 0;
+map_tile::map_tile() : iParcelId(0), iRoomId(0), flags({}), objects{}, raw{} {
+  tile_layers[tile_layer::ground] = 0;
+  tile_layers[tile_layer::north_wall] = 0;
+  tile_layers[tile_layer::west_wall] = 0;
+  tile_layers[tile_layer::ui] = 0;
   aiTemperature[0] = aiTemperature[1] = 8192;
 }
 
 level_map::level_map()
     : cells(nullptr),
       original_cells(nullptr),
-      blocks(nullptr),
+      wall_blocks(nullptr),
       overlay(nullptr),
       owns_overlay(false),
       plot_owner(nullptr),
@@ -380,7 +382,8 @@ bool level_map::load_blank() {
   map_tile* pOriginalNode = original_cells;
   for (int iY = 0; iY < height; ++iY) {
     for (int iX = 0; iX < width; ++iX, ++pNode, ++pOriginalNode) {
-      pNode->iBlock[0] = static_cast<uint16_t>(2 + (iX % 2));
+      pNode->tile_layers[tile_layer::ground] =
+          static_cast<uint16_t>(2 + (iX % 2));
     }
   }
   plot_owner = new int[1];
@@ -432,6 +435,7 @@ bool level_map::load_from_th_file(const uint8_t* pData, size_t iDataLength,
   pNode->objects.clear();
   for (int iY = 0; iY < height; ++iY) {
     for (int iX = 0; iX < width; ++iX) {
+      std::copy_n(pData, map_tile::raw_length, pNode->raw);
       uint8_t iBaseTile = gs_iTHMapBlockLUT[pData[2]];
       pNode->flags.can_travel_n = true;
       pNode->flags.can_travel_e = true;
@@ -449,28 +453,29 @@ bool level_map::load_from_th_file(const uint8_t* pData, size_t iDataLength,
         pNode->flags.can_travel_s = false;
       }
 
-      pNode->iBlock[0] = iBaseTile;
+      pNode->tile_layers[tile_layer::ground] = iBaseTile;
       if (pData[3] == 0 || is_divider_wall(pData[3])) {
         // Tiles 71, 72 and 73 (pond foliage) are used as floor tiles,
         // but are too tall to be floor tiles, so move them to a wall,
         // and replace the floor with something similar (pond base).
         if (71 <= iBaseTile && iBaseTile <= 73) {
-          pNode->iBlock[1] = iBaseTile;
-          pNode->iBlock[0] = 69;
+          pNode->tile_layers[tile_layer::north_wall] = iBaseTile;
+          pNode->tile_layers[tile_layer::ground] = 69;
         } else {
-          pNode->iBlock[1] = 0;
+          pNode->tile_layers[tile_layer::north_wall] = 0;
         }
       } else {
-        pNode->iBlock[1] = gs_iTHMapBlockLUT[pData[3]];
+        pNode->tile_layers[tile_layer::north_wall] =
+            gs_iTHMapBlockLUT[pData[3]];
         pNode->flags.can_travel_n = false;
         if (iY != 0) {
           pNode[-this->width].flags.can_travel_s = false;
         }
       }
       if (pData[4] == 0 || is_divider_wall(pData[4])) {
-        pNode->iBlock[2] = 0;
+        pNode->tile_layers[tile_layer::west_wall] = 0;
       } else {
-        pNode->iBlock[2] = gs_iTHMapBlockLUT[pData[4]];
+        pNode->tile_layers[tile_layer::west_wall] = gs_iTHMapBlockLUT[pData[4]];
         pNode->flags.can_travel_w = false;
         if (iX != 0) {
           pNode[-1].flags.can_travel_e = false;
@@ -507,10 +512,12 @@ bool level_map::load_from_th_file(const uint8_t* pData, size_t iDataLength,
 
       *pOriginalNode = *pNode;
       if (is_divider_wall(pData[3])) {
-        pOriginalNode->iBlock[1] = gs_iTHMapBlockLUT[pData[3]];
+        pOriginalNode->tile_layers[tile_layer::north_wall] =
+            gs_iTHMapBlockLUT[pData[3]];
       }
       if (is_divider_wall(pData[4])) {
-        pOriginalNode->iBlock[2] = gs_iTHMapBlockLUT[pData[4]];
+        pOriginalNode->tile_layers[tile_layer::west_wall] =
+            gs_iTHMapBlockLUT[pData[4]];
       }
 
       if (pData[1] != 0 && fnObjectCallback != nullptr) {
@@ -567,9 +574,12 @@ void level_map::save(const std::string& filename) {
                                                     : pNode->objects.front());
 
     // Blocks
-    aBuffer[iBufferNext++] = aReverseBlockLUT[pNode->iBlock[0] & 0xFF];
-    aBuffer[iBufferNext++] = aReverseBlockLUT[pNode->iBlock[1] & 0xFF];
-    aBuffer[iBufferNext++] = aReverseBlockLUT[pNode->iBlock[2] & 0xFF];
+    aBuffer[iBufferNext++] =
+        aReverseBlockLUT[pNode->tile_layers[tile_layer::ground] & 0xFF];
+    aBuffer[iBufferNext++] =
+        aReverseBlockLUT[pNode->tile_layers[tile_layer::north_wall] & 0xFF];
+    aBuffer[iBufferNext++] =
+        aReverseBlockLUT[pNode->tile_layers[tile_layer::west_wall] & 0xFF];
 
     // Flags (TODO: Set a few more flag bits?)
     uint8_t iFlags = 63;
@@ -649,17 +659,17 @@ namespace {
 */
 bool addRemoveDividerWalls(level_map* pMap, map_tile* pNode,
                            const map_tile* pOriginalNode, int iXY, int delta,
-                           int block, int iParcelId) {
+                           tile_layer layer, int iParcelId) {
   if (iXY > 0 && pOriginalNode->flags.hospital &&
       pOriginalNode[-delta].flags.hospital &&
       pNode->iParcelId != pNode[-delta].iParcelId) {
     int iOwner = pMap->get_parcel_owner(pNode->iParcelId);
     int iOtherOwner = pMap->get_parcel_owner(pNode[-delta].iParcelId);
     if (iOwner != iOtherOwner) {
-      pNode->iBlock[block] = block + (iOwner ? 143 : 141);
+      pNode->tile_layers[layer] = layer + (iOwner ? 143 : 141);
     } else if (pNode->iParcelId == iParcelId ||
                pNode[-delta].iParcelId == iParcelId) {
-      pNode->iBlock[block] = 0;
+      pNode->tile_layers[layer] = 0;
       return true;
     }
   }
@@ -683,31 +693,36 @@ std::vector<std::pair<int, int>> level_map::set_parcel_owner(int iParcelId,
     for (int iX = 0; iX < this->width; ++iX, ++pNode, ++pOriginalNode) {
       if (pNode->iParcelId == iParcelId) {
         if (iOwner != 0) {
-          pNode->iBlock[0] = pOriginalNode->iBlock[0];
-          pNode->iBlock[1] = pOriginalNode->iBlock[1];
-          pNode->iBlock[2] = pOriginalNode->iBlock[2];
+          pNode->tile_layers[tile_layer::ground] =
+              pOriginalNode->tile_layers[tile_layer::ground];
+          pNode->tile_layers[tile_layer::north_wall] =
+              pOriginalNode->tile_layers[tile_layer::north_wall];
+          pNode->tile_layers[tile_layer::west_wall] =
+              pOriginalNode->tile_layers[tile_layer::west_wall];
           pNode->flags = pOriginalNode->flags;
         } else {
           // Nicely mown grass pattern
-          pNode->iBlock[0] = static_cast<uint16_t>(((iX & 1) << 1) + 1);
+          pNode->tile_layers[tile_layer::ground] =
+              static_cast<uint16_t>(((iX & 1) << 1) + 1);
 
-          pNode->iBlock[1] = 0;
-          pNode->iBlock[2] = 0;
+          pNode->tile_layers[tile_layer::north_wall] = 0;
+          pNode->tile_layers[tile_layer::west_wall] = 0;
           pNode->flags = {};
 
           // Random decoration
           if (((iX | iY) & 0x7) == 0) {
             int iWhich = (iX ^ iY) % 9;
-            pNode->iBlock[1] = static_cast<uint16_t>(192 + iWhich);
+            pNode->tile_layers[tile_layer::north_wall] =
+                static_cast<uint16_t>(192 + iWhich);
           }
         }
       }
-      if (addRemoveDividerWalls(this, pNode, pOriginalNode, iX, 1, 2,
-                                iParcelId)) {
+      if (addRemoveDividerWalls(this, pNode, pOriginalNode, iX, 1,
+                                tile_layer::west_wall, iParcelId)) {
         vSplitTiles.push_back(std::make_pair(iX, iY));
       }
-      if (addRemoveDividerWalls(this, pNode, pOriginalNode, iY, this->width, 1,
-                                iParcelId)) {
+      if (addRemoveDividerWalls(this, pNode, pOriginalNode, iY, this->width,
+                                tile_layer::north_wall, iParcelId)) {
         vSplitTiles.push_back(std::make_pair(iX, iY));
       }
     }
@@ -936,16 +951,16 @@ const map_tile* level_map::get_original_tile_unchecked(int iX, int iY) const {
   return original_cells + iY * width + iX;
 }
 
-void level_map::set_block_sheet(sprite_sheet* pSheet) { blocks = pSheet; }
+void level_map::set_block_sheet(sprite_sheet* pSheet) { wall_blocks = pSheet; }
 
 void level_map::set_all_wall_draw_flags(uint8_t iFlags) {
-  uint16_t iBlockOr = static_cast<uint16_t>(iFlags << 8);
+  uint16_t draw_flags = static_cast<uint16_t>(iFlags << 8);
   map_tile* pNode = cells;
   for (int i = 0; i < width * height; ++i, ++pNode) {
-    pNode->iBlock[1] =
-        static_cast<uint16_t>((pNode->iBlock[1] & 0xFF) | iBlockOr);
-    pNode->iBlock[2] =
-        static_cast<uint16_t>((pNode->iBlock[2] & 0xFF) | iBlockOr);
+    pNode->tile_layers[tile_layer::north_wall] = static_cast<uint16_t>(
+        (pNode->tile_layers[tile_layer::north_wall] & 0xFF) | draw_flags);
+    pNode->tile_layers[tile_layer::west_wall] = static_cast<uint16_t>(
+        (pNode->tile_layers[tile_layer::west_wall] & 0xFF) | draw_flags);
   }
 }
 
@@ -971,7 +986,7 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
       2) For each tile, left to right, the west wall, then the late entities
   */
 
-  if (blocks == nullptr || cells == nullptr) {
+  if (wall_blocks == nullptr || cells == nullptr) {
     return;
   }
 
@@ -986,26 +1001,26 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
        itrNode1; ++itrNode1) {
     // First, draw the floor tile as it should be below everything else.
     int iH = 32;
-    unsigned int iBlock = itrNode1->iBlock[0];
-    blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH);
-    blocks->draw_sprite(
-        pCanvas, iBlock & 0xFF,
+    uint16_t layer = itrNode1->tile_layers[tile_layer::ground];
+    wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH);
+    wall_blocks->draw_sprite(
+        pCanvas, layer & 0xFF,
         itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
         itrNode1.tile_y_position_on_screen() + iCanvasY - iH + 32,
-        (iBlock >> 8) | thdf_nearest);
+        (layer >> 8) | thdf_nearest);
 
     // Draw floor shadows immediately after floor tiles ensuring that all
     // shadow pixels are drawn onto freshly drawn opaque floor tile pixels.
     if (itrNode1->flags.shadow_full) {
-      blocks->draw_sprite(pCanvas, 74,
-                          itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
-                          itrNode1.tile_y_position_on_screen() + iCanvasY,
-                          thdf_alpha_75 | thdf_nearest);
+      wall_blocks->draw_sprite(
+          pCanvas, 74, itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
+          itrNode1.tile_y_position_on_screen() + iCanvasY,
+          thdf_alpha_75 | thdf_nearest);
     } else if (itrNode1->flags.shadow_half) {
-      blocks->draw_sprite(pCanvas, 75,
-                          itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
-                          itrNode1.tile_y_position_on_screen() + iCanvasY,
-                          thdf_alpha_75 | thdf_nearest);
+      wall_blocks->draw_sprite(
+          pCanvas, 75, itrNode1.tile_x_position_on_screen() + iCanvasX - 32,
+          itrNode1.tile_y_position_on_screen() + iCanvasY,
+          thdf_alpha_75 | thdf_nearest);
     }
   }
 
@@ -1022,12 +1037,12 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
              iCanvasY);
          itrNode; ++itrNode) {
       int iH;
-      unsigned int iBlock = itrNode->iBlock[1];
-      if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
-          iH > 0) {
-        blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32,
-                            (iBlock >> 8) | thdf_nearest);
+      uint16_t layer = itrNode->tile_layers[tile_layer::north_wall];
+      if (layer != 0 &&
+          wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH) && iH > 0) {
+        wall_blocks->draw_sprite(pCanvas, layer & 0xFF, itrNode.x() - 32,
+                                 itrNode.y() - iH + 32,
+                                 (layer >> 8) | thdf_nearest);
         if (itrNode->flags.shadow_wall) {
           clip_rect rcNewClip;
           rcNewClip.x = static_cast<clip_rect::x_y_type>(itrNode.x() - 32);
@@ -1036,13 +1051,14 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
           rcNewClip.w = static_cast<clip_rect::w_h_type>(64);
           rcNewClip.h = static_cast<clip_rect::w_h_type>(86 - 4);
           render_target::scoped_clip clip(pCanvas, &rcNewClip);
-          blocks->draw_sprite(pCanvas, 156, itrNode.x() - 32, itrNode.y() - 56,
-                              thdf_alpha_75 | thdf_nearest);
+          wall_blocks->draw_sprite(pCanvas, 156, itrNode.x() - 32,
+                                   itrNode.y() - 56,
+                                   thdf_alpha_75 | thdf_nearest);
         }
       }
       drawable* pItem = static_cast<drawable*>(itrNode->oEarlyEntities.next);
       while (pItem) {
-        pItem->draw_fn(pItem, pCanvas, itrNode.x(), itrNode.y());
+        pItem->draw_fn(pCanvas, itrNode.x(), itrNode.y());
         pItem = static_cast<drawable*>(pItem->next);
       }
     }
@@ -1064,23 +1080,23 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
     for (; itrNode; ++itrNode) {
       bool bNeedsRedraw = false;
       int iH;
-      unsigned int iBlock = itrNode->iBlock[2];
-      if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
-          iH > 0) {
-        blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32,
-                            (iBlock >> 8) | thdf_nearest);
+      uint16_t layer = itrNode->tile_layers[tile_layer::west_wall];
+      if (layer != 0 &&
+          wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH) && iH > 0) {
+        wall_blocks->draw_sprite(pCanvas, layer & 0xFF, itrNode.x() - 32,
+                                 itrNode.y() - iH + 32,
+                                 (layer >> 8) | thdf_nearest);
       }
-      iBlock = itrNode->iBlock[3];
-      if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
-          iH > 0) {
-        blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 32,
-                            itrNode.y() - iH + 32,
-                            (iBlock >> 8) | thdf_nearest);
+      layer = itrNode->tile_layers[tile_layer::ui];
+      if (layer != 0 &&
+          wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH) && iH > 0) {
+        wall_blocks->draw_sprite(pCanvas, layer & 0xFF, itrNode.x() - 32,
+                                 itrNode.y() - iH + 32,
+                                 (layer >> 8) | thdf_nearest);
       }
-      iBlock = itrNode->iBlock[1];
-      if (iBlock != 0 && blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) &&
-          iH > 0) {
+      layer = itrNode->tile_layers[tile_layer::north_wall];
+      if (layer != 0 &&
+          wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH) && iH > 0) {
         bNeedsRedraw = true;
       }
       if (itrNode->oEarlyEntities.next) {
@@ -1091,8 +1107,8 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
 
       drawable* pItem = static_cast<drawable*>(itrNode->entities.next);
       while (pItem) {
-        pItem->draw_fn(pItem, pCanvas, itrNode.x(), itrNode.y());
-        if (pItem->is_multiple_frame_animation_fn(pItem)) {
+        pItem->draw_fn(pCanvas, itrNode.x(), itrNode.y());
+        if (pItem->is_multiple_frame_animation_fn()) {
           bRedrawAnimations = true;
         }
         if (pItem->get_drawing_layer() == 1) {
@@ -1115,7 +1131,7 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
             formerIterator.get_previous_tile()->entities.next);
         while (pItem) {
           if (pItem->get_drawing_layer() == 9) {
-            pItem->draw_fn(pItem, pCanvas, formerIterator.x() - 64,
+            pItem->draw_fn(pCanvas, formerIterator.x() - 64,
                            formerIterator.y());
             bTileNeedsRedraw = true;
           }
@@ -1129,8 +1145,7 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
                     : nullptr;
         while (pItem) {
           if (pItem->get_drawing_layer() == 8) {
-            pItem->draw_fn(pItem, pCanvas, formerIterator.x(),
-                           formerIterator.y());
+            pItem->draw_fn(pCanvas, formerIterator.x(), formerIterator.y());
           }
           pItem = static_cast<drawable*>(pItem->next);
         }
@@ -1140,12 +1155,14 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
         // north side or a wall to the north redraw that tile
         if (bTileNeedsRedraw) {
           // redraw the north wall
-          unsigned int iBlock = itrNode.get_previous_tile()->iBlock[1];
-          if (iBlock != 0 &&
-              blocks->get_sprite_size(iBlock & 0xFF, nullptr, &iH) && iH > 0) {
-            blocks->draw_sprite(pCanvas, iBlock & 0xFF, itrNode.x() - 96,
-                                itrNode.y() - iH + 32,
-                                (iBlock >> 8) | thdf_nearest);
+          uint16_t layer =
+              itrNode.get_previous_tile()->tile_layers[tile_layer::north_wall];
+          if (layer != 0 &&
+              wall_blocks->get_sprite_size(layer & 0xFF, nullptr, &iH) &&
+              iH > 0) {
+            wall_blocks->draw_sprite(pCanvas, layer & 0xFF, itrNode.x() - 96,
+                                     itrNode.y() - iH + 32,
+                                     (layer >> 8) | thdf_nearest);
             if (itrNode.get_previous_tile()->flags.shadow_wall) {
               clip_rect rcNewClip;
               rcNewClip.x = static_cast<clip_rect::x_y_type>(itrNode.x() - 96);
@@ -1154,22 +1171,22 @@ void level_map::draw(render_target* pCanvas, int iScreenX, int iScreenY,
               rcNewClip.w = static_cast<clip_rect::w_h_type>(64);
               rcNewClip.h = static_cast<clip_rect::w_h_type>(86 - 4);
               render_target::scoped_clip clip(pCanvas, &rcNewClip);
-              blocks->draw_sprite(pCanvas, 156, itrNode.x() - 96,
-                                  itrNode.y() - 56,
-                                  thdf_alpha_75 | thdf_nearest);
+              wall_blocks->draw_sprite(pCanvas, 156, itrNode.x() - 96,
+                                       itrNode.y() - 56,
+                                       thdf_alpha_75 | thdf_nearest);
             }
           }
 
           pItem = static_cast<drawable*>(
               itrNode.get_previous_tile()->oEarlyEntities.next);
           for (; pItem; pItem = static_cast<drawable*>(pItem->next)) {
-            pItem->draw_fn(pItem, pCanvas, itrNode.x() - 64, itrNode.y());
+            pItem->draw_fn(pCanvas, itrNode.x() - 64, itrNode.y());
           }
 
           pItem = static_cast<drawable*>(
               itrNode.get_previous_tile()->entities.next);
           for (; pItem; pItem = static_cast<drawable*>(pItem->next)) {
-            pItem->draw_fn(pItem, pCanvas, itrNode.x() - 64, itrNode.y());
+            pItem->draw_fn(pCanvas, itrNode.x() - 64, itrNode.y());
           }
         }
       }
@@ -1198,7 +1215,7 @@ drawable* level_map::hit_test(int iTestX, int iTestY) const {
   // This function needs to hitTest each drawable object, in the reverse
   // order to that in which they would be drawn.
 
-  if (blocks == nullptr || cells == nullptr) {
+  if (wall_blocks == nullptr || cells == nullptr) {
     return nullptr;
   }
 
@@ -1245,7 +1262,7 @@ drawable* level_map::hit_test_drawables(link_list* pListStart, int iXs, int iYs,
   drawable* pList = static_cast<drawable*>(pListEnd);
 
   while (true) {
-    if (pList->hit_test_fn(pList, iXs, iYs, iTestX, iTestY)) return pList;
+    if (pList->hit_test_fn(iXs, iYs, iTestX, iTestY)) return pList;
 
     if (pList == pListStart) {
       return nullptr;
@@ -1408,13 +1425,13 @@ void level_map::update_pathfinding() {
         pNode->flags.can_travel_s = false;
       }
 
-      if (pNode->iBlock[1] & 0xFF) {
+      if (pNode->tile_layers[tile_layer::north_wall] & 0xFF) {
         pNode->flags.can_travel_n = false;
         if (iY != 0) {
           pNode[-this->width].flags.can_travel_s = false;
         }
       }
-      if (pNode->iBlock[2] & 0xFF) {
+      if (pNode->tile_layers[tile_layer::west_wall] & 0xFF) {
         pNode->flags.can_travel_w = false;
         if (iX != 0) {
           pNode[-1].flags.can_travel_e = false;
@@ -1429,9 +1446,9 @@ namespace {
 //! For shadow casting, a tile is considered to have a wall on a direction
 //! if it has a door in that direction, or the block is from the hardcoded
 //! range of wall-like blocks.
-bool is_wall(map_tile* tile, size_t block, bool flag) {
-  return flag || (82 <= (tile->iBlock[block] & 0xFF) &&
-                  (tile->iBlock[block] & 0xFF) <= 164);
+bool is_wall(map_tile* tile, tile_layer layer, bool flag) {
+  return flag || (82 <= (tile->tile_layers[layer] & 0xFF) &&
+                  (tile->tile_layers[layer] & 0xFF) <= 164);
 }
 
 }  // namespace
@@ -1443,14 +1460,15 @@ void level_map::update_shadows() {
       pNode->flags.shadow_full = false;
       pNode->flags.shadow_half = false;
       pNode->flags.shadow_wall = false;
-      if (is_wall(pNode, 2, pNode->flags.tall_west)) {
+      if (is_wall(pNode, tile_layer::west_wall, pNode->flags.tall_west)) {
         pNode->flags.shadow_half = true;
-        if (is_wall(pNode, 1, pNode->flags.tall_north)) {
+        if (is_wall(pNode, tile_layer::north_wall, pNode->flags.tall_north)) {
           pNode->flags.shadow_wall = true;
         } else if (iY != 0) {
           map_tile* pNeighbour = pNode - this->width;
           pNeighbour->flags.shadow_full = true;
-          if (iX != 0 && !is_wall(pNeighbour, 2, pNode->flags.tall_west)) {
+          if (iX != 0 && !is_wall(pNeighbour, tile_layer::west_wall,
+                                  pNode->flags.tall_west)) {
             // Wrap the shadow around a corner (no need to continue
             // all the way along the wall, as the shadow would be
             // occluded by the wall. If Debug->Transparent Walls is
@@ -1469,7 +1487,7 @@ void level_map::persist(lua_persist_writer* pWriter) const {
   lua_State* L = pWriter->get_stack();
   integer_run_length_encoder oEncoder;
 
-  uint32_t iVersion = 4;
+  uint32_t iVersion = 5;
   pWriter->write_uint(iVersion);
   pWriter->write_uint(player_count);
   for (int i = 0; i < player_count; ++i) {
@@ -1491,10 +1509,10 @@ void level_map::persist(lua_persist_writer* pWriter) const {
   oEncoder.initialise(6);
   for (map_tile *pNode = cells, *pLimitNode = cells + width * height;
        pNode != pLimitNode; ++pNode) {
-    oEncoder.write(pNode->iBlock[0]);
-    oEncoder.write(pNode->iBlock[1]);
-    oEncoder.write(pNode->iBlock[2]);
-    oEncoder.write(pNode->iBlock[3]);
+    oEncoder.write(pNode->tile_layers[tile_layer::ground]);
+    oEncoder.write(pNode->tile_layers[tile_layer::north_wall]);
+    oEncoder.write(pNode->tile_layers[tile_layer::west_wall]);
+    oEncoder.write(pNode->tile_layers[tile_layer::ui]);
     oEncoder.write(pNode->iParcelId);
     oEncoder.write(pNode->iRoomId);
     // Flags include THOB values, and other things which do not work
@@ -1502,6 +1520,7 @@ void level_map::persist(lua_persist_writer* pWriter) const {
     pWriter->write_uint(static_cast<uint32_t>(pNode->flags));
     pWriter->write_uint(pNode->aiTemperature[0]);
     pWriter->write_uint(pNode->aiTemperature[1]);
+    pWriter->write_byte_stream(pNode->raw, map_tile::raw_length);
 
     lua_rawgeti(L, luaT_upvalueindex(1), 2);
     lua_pushlightuserdata(L, pNode->entities.next);
@@ -1520,9 +1539,9 @@ void level_map::persist(lua_persist_writer* pWriter) const {
   for (map_tile *pNode = original_cells,
                 *pLimitNode = original_cells + width * height;
        pNode != pLimitNode; ++pNode) {
-    oEncoder.write(pNode->iBlock[0]);
-    oEncoder.write(pNode->iBlock[1]);
-    oEncoder.write(pNode->iBlock[2]);
+    oEncoder.write(pNode->tile_layers[tile_layer::ground]);
+    oEncoder.write(pNode->tile_layers[tile_layer::north_wall]);
+    oEncoder.write(pNode->tile_layers[tile_layer::west_wall]);
     oEncoder.write(pNode->iParcelId);
     oEncoder.write(static_cast<uint32_t>(pNode->flags));
   }
@@ -1539,14 +1558,12 @@ void level_map::depersist(lua_persist_reader* pReader) {
 
   uint32_t iVersion;
   if (!pReader->read_uint(iVersion)) return;
-  if (iVersion != 4) {
-    if (iVersion < 2 || iVersion == 128) {
-      luaL_error(L,
-                 "TODO: Write code to load map data from earlier "
-                 "savegame versions (if really necessary).");
-    } else if (iVersion > 4) {
-      luaL_error(L, "Cannot load savegame from a newer version.");
-    }
+  if (iVersion < 2 || iVersion == 128) {
+    luaL_error(L,
+               "TODO: Write code to load map data from earlier "
+               "savegame versions (if really necessary).");
+  } else if (iVersion > 5) {
+    luaL_error(L, "Cannot load savegame from a newer version.");
   }
   if (!pReader->read_uint(player_count)) return;
   for (int i = 0; i < player_count; ++i) {
@@ -1602,9 +1619,12 @@ void level_map::depersist(lua_persist_reader* pReader) {
         return;
       }
     }
+    if (iVersion >= 5) {
+      if (!pReader->read_byte_stream(pNode->raw, map_tile::raw_length)) return;
+    }
 
     if (!pReader->read_stack_object()) return;
-    pNode->entities.next = static_cast<link_list*>(lua_touserdata(L, -1));
+    pNode->entities.next = luaT_toanimationbase(L, -1);
     if (pNode->entities.next) {
       if (pNode->entities.next->prev != nullptr) {
         std::fprintf(stderr, "Warning: THMap linked-lists are corrupted.\n");
@@ -1614,7 +1634,7 @@ void level_map::depersist(lua_persist_reader* pReader) {
     lua_pop(L, 1);
 
     if (!pReader->read_stack_object()) return;
-    pNode->oEarlyEntities.next = static_cast<link_list*>(lua_touserdata(L, -1));
+    pNode->oEarlyEntities.next = luaT_toanimationbase(L, -1);
     if (pNode->oEarlyEntities.next) {
       if (pNode->oEarlyEntities.next->prev != nullptr) {
         std::fprintf(stderr, "Warning: THMap linked-lists are corrupted.\n");
@@ -1626,10 +1646,13 @@ void level_map::depersist(lua_persist_reader* pReader) {
   oDecoder.initialise(6, pReader);
   for (map_tile *pNode = cells, *pLimitNode = cells + width * height;
        pNode != pLimitNode; ++pNode) {
-    pNode->iBlock[0] = static_cast<uint16_t>(oDecoder.read());
-    pNode->iBlock[1] = static_cast<uint16_t>(oDecoder.read());
-    pNode->iBlock[2] = static_cast<uint16_t>(oDecoder.read());
-    pNode->iBlock[3] = static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::ground] =
+        static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::north_wall] =
+        static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::west_wall] =
+        static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::ui] = static_cast<uint16_t>(oDecoder.read());
     pNode->iParcelId = static_cast<uint16_t>(oDecoder.read());
     pNode->iRoomId = static_cast<uint16_t>(oDecoder.read());
   }
@@ -1637,9 +1660,12 @@ void level_map::depersist(lua_persist_reader* pReader) {
   for (map_tile *pNode = original_cells,
                 *pLimitNode = original_cells + width * height;
        pNode != pLimitNode; ++pNode) {
-    pNode->iBlock[0] = static_cast<uint16_t>(oDecoder.read());
-    pNode->iBlock[1] = static_cast<uint16_t>(oDecoder.read());
-    pNode->iBlock[2] = static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::ground] =
+        static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::north_wall] =
+        static_cast<uint16_t>(oDecoder.read());
+    pNode->tile_layers[tile_layer::west_wall] =
+        static_cast<uint16_t>(oDecoder.read());
     pNode->iParcelId = static_cast<uint16_t>(oDecoder.read());
     pNode->flags = oDecoder.read();
   }
