@@ -22,10 +22,21 @@ SOFTWARE.
 
 #ifndef CORSIX_TH_TH_MAP_H_
 #define CORSIX_TH_TH_MAP_H_
+#include "config.h"
+
 #include <list>
 #include <string>
+#include <utility>
+#include <vector>
 
-#include "th_gfx.h"
+#include "th.h"
+
+class lua_persist_reader;
+class lua_persist_writer;
+class map_overlay;
+class render_target;
+class sprite_sheet;
+struct drawable;
 
 /*
     Object type enumeration uses same values as original TH does.
@@ -171,6 +182,15 @@ enum class temperature_theme {
   yellow_red     //!< Gradients of yellow, orange, and red
 };
 
+enum tile_layer : uint8_t {
+  ground = 0,
+  north_wall = 1,
+  west_wall = 2,
+  ui = 3,
+
+  num_tile_layers = 4  //!< Number of layers in a tile.
+};
+
 struct map_tile {
   map_tile();
   ~map_tile() = default;
@@ -181,15 +201,15 @@ struct map_tile {
   //! Linked list for entities rendered in an early (right-to-left) pass
   link_list oEarlyEntities;
 
-  //! Block tiles for rendering
-  //! For each tile, the lower byte is the index in the sprite sheet, and the
+  //! Tile layers for rendering.
+  //! For each layer, the lower byte is the index in the sprite sheet, and the
   //! upper byte is for the drawing flags.
   //! Layer 0 is for the floor
   //! Layer 1 is for the north wall
   //! Layer 2 is for the west wall
   //! Layer 3 is for the UI
   //! NB: In Lua, layers are numbered 1 - 4 rather than 0 - 3
-  uint16_t iBlock[4];
+  uint16_t tile_layers[tile_layer::num_tile_layers];
 
   //! Parcels (plots) of land have an ID, with each tile in the plot having
   //! that ID. Parcel 0 is the outside.
@@ -210,9 +230,12 @@ struct map_tile {
 
   //! objects in this tile
   std::list<object_type> objects;
-};
 
-class sprite_sheet;
+  static constexpr int raw_length = 8;
+
+  //! raw data from map file for debugging
+  std::uint8_t raw[raw_length];
+};
 
 //! Prototype for object callbacks from THMap::loadFromTHFile
 /*!
@@ -226,8 +249,6 @@ class sprite_sheet;
 typedef void (*map_load_object_callback_fn)(void*, int, int, object_type,
                                             uint8_t);
 
-class map_overlay;
-
 class level_map {
  public:
   level_map();
@@ -413,7 +434,7 @@ class level_map {
 
   map_tile* cells;
   map_tile* original_cells;  // Cells at map load time, before any changes
-  sprite_sheet* blocks;
+  sprite_sheet* wall_blocks;
   map_overlay* overlay;
   bool owns_overlay;
   int* plot_owner;  // 0 for unowned, 1 for player 1, etc.
