@@ -27,21 +27,31 @@ SOFTWARE.
 #include "lua_sdl.h"
 #if defined(CORSIX_TH_USE_FFMPEG) && defined(CORSIX_TH_USE_SDL_MIXER)
 
-#include "th_gfx.h"
 extern "C" {
 #include <libavcodec/avcodec.h>
 #include <libavutil/avutil.h>
 #include <libavutil/channel_layout.h>
+#include <libavutil/error.h>
 #include <libavutil/imgutils.h>
-#include <libavutil/mathematics.h>
-#include <libavutil/opt.h>
+#include <libavutil/rational.h>
+#include <libavutil/samplefmt.h>
+#include <libswresample/version.h>
 #include <libswscale/swscale.h>
 }
+#include <SDL_error.h>
+#include <SDL_events.h>
 #include <SDL_mixer.h>
+#include <SDL_pixels.h>
+#include <SDL_rect.h>
+#include <SDL_render.h>
+#include <SDL_timer.h>
 
+#include <cerrno>
 #include <chrono>
 #include <cstring>
 #include <iostream>
+#include <stdexcept>
+#include <utility>
 
 namespace {
 
@@ -612,7 +622,7 @@ void movie_player::run_video() {
       break;
     } else if (iError < 0) {
       std::cerr << "Unexpected error " << iError
-                << " while decoding video packet" << std::endl;
+                << " while decoding video packet\n";
       break;
     }
 
@@ -699,7 +709,7 @@ int movie_player::decode_audio_frame(uint8_t* stream, int stream_size) {
       swr_convert(audio_resample_context, &stream, iOutSamples, nullptr, 0);
   if (actual_samples < 0) {
     std::cerr << "WARN: Unexpected error " << actual_samples
-              << " while converting audio" << std::endl;
+              << " while converting audio\n";
     return 0;
   } else if (actual_samples > 0) {
     return actual_samples * av_get_bytes_per_sample(AV_SAMPLE_FMT_S16) *
@@ -713,7 +723,7 @@ int movie_player::decode_audio_frame(uint8_t* stream, int stream_size) {
     return 0;
   } else if (iError < 0) {
     std::cerr << "WARN: Unexpected error " << iError
-              << " while decoding audio packet" << std::endl;
+              << " while decoding audio packet\n";
     return 0;
   }
 
@@ -728,7 +738,7 @@ int movie_player::decode_audio_frame(uint8_t* stream, int stream_size) {
                   audio_frame->nb_samples);
   if (actual_samples < 0) {
     std::cerr << "WARN: Unexpected error " << actual_samples
-              << " while converting audio" << std::endl;
+              << " while converting audio\n";
     return 0;
   }
   return actual_samples * av_get_bytes_per_sample(AV_SAMPLE_FMT_S16) *
